if exists (select 1 from sysobjects where name='DBA_CreateDefaultConstraint' and type='P')
   drop procedure DBA_CreateDefaultConstraint  
go
Create  Procedure DBA_CreateDefaultConstraint  
   @pvcTableName Varchar(80),  
   @pvcSchemaName Varchar(80),
   @pvcColumnName Varchar(80),
   @pvcDefaultValue Varchar(80),  
   @pvcConstraintName Varchar(100)=NULL  
AS 
---------------------------------------------------------------------------------------
-- Stored Procedure to Create a default constraint.
-- Required Parms: TableName, ColumnName, DefaultValue
-- Optional Parms: ConstraintName 
--    If Identical constraint exists with same name - no action taken.
--    If Identical constraint exists with different name - constraint is renamed.
--    If already-existing constraint with diff attributes - drop it and recreate it
--


SET NOCOUNT ON
Print '-------------------------------------------------------------------------------'
Print 'Calling DBA_CreateDefaultConstraint with '+@pvcTableName+'.'+@pvcColumnName+' ConstName='+ISNULL(@pvcConstraintName,'N/A')+' Value='+@pvcDefaultValue
Print '-------------------------------------------------------------------------------'
declare @SQL varchar(1000)
declare @iConstraintNameSpecified smallint

declare @vcConstraintName Varchar(100)
declare @vcDefaultValue varchar(100)
If @pvcConstraintName is NULL 
   SET @iConstraintNameSpecified=0
ELSE 
   SET @iConstraintNameSpecified=1


-------------------------------------------------
-- Check if exact constraint exists 
-- With same name (disregard name if parm is NULL
-------------------------------------------------
SELECT @vcConstraintName=NULL
SELECT @vcConstraintName=object_name(con.constid)
   from sysconstraints con
   inner join syscolumns col  
   on con.id = col.id   
   and col.colid = con.colid   
   inner join syscomments com
   on col.cdefault=com.id
   inner join sysobjects obj
   on con.constid = obj.id and obj.xtype='D'
inner join sys.objects so
on obj.id = so.object_id
   where col.name = @pvcColumnName  
and so.schema_id = schema_id(@pvcSchemaName)
   and object_name(con.id) = @pvcTableName  
   and object_name(col.id) = @pvcTableName 
   and com.text=@pvcDefaultValue
   and (@iConstraintNameSpecified=0 OR object_name(con.constid)=@pvcConstraintName)
IF @vcConstraintName IS NOT NULL
BEGIN
   PRINT '  Specified constraint already exists.  No action performed.'
   goto DoneProcessing
END



-------------------------------------------------
-- Check if exact constraint exists 
-- but with a DIFFERENT name
-------------------------------------------------
SELECT @vcConstraintName=NULL
SELECT @vcConstraintName=object_name(con.constid)
   from sysconstraints con
   inner join syscolumns col  
   on con.id = col.id   
   and col.colid = con.colid   
   inner join syscomments com
   on col.cdefault=com.id
   inner join sysobjects obj
   on con.constid = obj.id and obj.xtype='D'
   where col.name = @pvcColumnName  

   and object_name(con.id) = @pvcTableName  
   and object_name(col.id) = @pvcTableName 
   and com.text=@pvcDefaultValue

IF @vcConstraintName IS NOT NULL
BEGIN
   --SELECT @SQL='EXEC sp_rename '''+@vcConstraintName+''''+', '''+@pvcConstraintName+''''
	--PRINT '  An identical Default Constraint already exists; but with a different name ('+@vcConstraintName+')'
   --PRINT '  Constraint will be renamed with the following command:'
   --PRINT '    '+@SQL
   --EXEC (@SQL)
   PRINT '  An identical Default Constraint already exists; but with a different name ('+@vcConstraintName+')'
   PRINT '  Constraint will be dropped with the following command:'
   SELECT @SQL = 'ALTER TABLE ' + quotename(@pvcSchemaName) + '.' + quotename(@pvcTableName) + ' DROP CONSTRAINT ' + quotename(@vcConstraintName)
   PRINT '    '+@SQL
   EXEC (@SQL)
  
   --goto DoneProcessing

END






-------------------------------------------------
-- Check if any constraint exists on column
-- With same name (disregard name if parm is NULL
-------------------------------------------------
SELECT @vcConstraintName=NULL
SELECT @vcConstraintName=object_name(con.constid)
	, @vcDefaultValue=com.text
   from sysconstraints con
   inner join syscolumns col  
   on con.id = col.id   
   and col.colid = con.colid   
   inner join syscomments com
   on col.cdefault=com.id
   inner join sysobjects obj
   on con.constid = obj.id and obj.xtype='D'
inner join sys.objects so
on obj.id = so.object_id
   where col.name = @pvcColumnName  
and so.schema_id = schema_id(@pvcSchemaName)
   and object_name(con.id) = @pvcTableName  
   and object_name(col.id) = @pvcTableName 
IF @vcConstraintName IS NOT NULL
BEGIN
   SELECT @SQL='ALTER TABLE ' + @pvcSchemaName + '.' +@pvcTableName+' DROP CONSTRAINT '+@vcConstraintName
	PRINT '  Default Constraint '+@vcConstraintName+' '+@vcDefaultValue+' already exists for specified column.'
   PRINT '  Constraint will be dropped with the following command:'
   PRINT '    '+@SQL
   EXEC (@SQL)
END

IF @pvcConstraintName IS NULL
   SET @pvcConstraintName = @pvcTableName+'_'+@pvcColumnName+'_DF'
SELECT @SQL='ALTER TABLE ' + @pvcSchemaName + '.' + @pvcTableName + ' WITH NOCHECK ADD CONSTRAINT ' + @pvcConstraintName  + ' DEFAULT '+@pvcDefaultValue+' FOR ' + @pvcColumnName  
PRINT '  Adding default constraint with the following command:'
PRINT '    '+@SQL
EXEC (@SQL)



DoneProcessing: