if exists (select * from sysobjects where id = object_id('DBA_CreateForeignKey'))
begin
	drop proc DBA_CreateForeignKey
end
go

CREATE  Procedure DBA_CreateForeignKey     
@pvcTableName Varchar(100),      

@pvcSchemaName Varchar(100),     

@pvcColumnName Varchar(100),       

@pvcConstraintName Varchar(100),         

@pvcParentTableName Varchar(100),    

@pvcParentSchemaName Varchar(100),    

@pvcParentKey varchar(100) ,  

@pvcDeleteAction varchar (100), --should be either CASCADE or NO_ACTION 

@pvcUpdateAction varchar (100) --should be either CASCADE or NO_ACTION 

    

AS      

---------------------------------------------------------------------------------------    

-- Stored Procedure to Create a foreign key    

-- Required Parms: TableName, ColumnName, ConstraintName, ParentTableName, ParentKey    

-- Optional Parms: None    

--    If Identical FK exists with same name - no action taken.    
--	  If Identical FK except cascading options, drop existing and recreate with new cascading options

--    If Identical FK with different name - FK is renamed.    

--    If already-existing constraint with diff attributes - rename it and create new FK    

--    

--select * from dba_helpfk ('tblLayer','dbo')  

--select * from dba_helpfk ('employeetimecarddetail','mxw')  

---------------------------------------------------------------------------------------    

    

SET NOCOUNT ON    

Print '-------------------------------------------------------------------------------'    

Print 'Calling DBA_CreateForeignKey with '+@pvcTableName+'.('+@pvcColumnName+')'    

Print '  FKName('+@pvcConstraintName+') '    

Print '  Referencing Parent '+@pvcParentTableName+'.('+@pvcParentKey +')'    

   

declare @SQL varchar(1000)    

declare @vcConstraintName Varchar(100)    

declare @cKeyIsMultiColumn char(1)    

    

--------------------------------------    

-- Check Parms    

--------------------------------------    

    

if charindex(',',@pvcColumnName) > 0    

BEGIN    

   SET @cKeyIsMultiColumn = 'Y'    

    

   DECLARE @iFKeyCount smallint    

   DECLARE @tFKeys TABLE (ColumnName varchar(100))    

   INSERT INTO @tFKeys SELECT ColumnName from dba_SplitStrings(@pvcColumnName)    

   SELECT @iFKeyCount=count(1) from @tFKeys    

    

   DECLARE @iPKeyCount smallint    

   DECLARE @tPKeys TABLE (ColumnName varchar(100))    

   INSERT INTO @tPKeys SELECT ColumnName from dba_SplitStrings(@pvcParentKey)    

   SELECT @iPKeyCount=count(1) from @tPKeys    

END    

ELSE     

   SET @cKeyIsMultiColumn = 'N'    

    

IF @cKeyIsMultiColumn = 'N'    

BEGIN    

   IF NOT EXISTS     

     (select 1    

     from sysobjects o    

     inner join syscolumns c    

     on o.id=c.id    

  inner join sys.objects so  

  on o.id = so.object_id  

     where o.name=@pvcTableName     

  and so.schema_id = schema_id (@pvcSchemaName)  

     and c.name=@pvcColumnName     

     )    

   BEGIN     

      Print '  Error Msg - Specified Table.Column1 ('+@pvcTableName+'.'+@pvcColumnName+') does not exist.  No action performed.'    

      goto DoneProcessing    

   END    

END --Single-column Key    

ELSE BEGIN --Multi-column Key    

   DECLARE @iFKeysInTable int    

    

   SELECT @iFKeysInTable = count(distinct t1.ColumnName)    

     from  sysobjects o    

     inner join syscolumns c    

     on o.id=c.id    

     inner join @tFKeys t1    

     on c.name = t1.ColumnName    

    

   if @iFKeysInTable <> @iFKeyCount    

   BEGIN     

      Print '  Error Msg - Specified Table.ColumnList ('+@pvcTableName+'.('+@pvcColumnName+') ) does not match.  One or more key does not exist.  No action performed.'    

      goto DoneProcessing    

   END    

END--Multi-column Key    

    

IF @cKeyIsMultiColumn = 'N'    

BEGIN    

   IF NOT EXISTS     

     (select 1    

     from sysobjects o    

     inner join syscolumns c    

     on o.id=c.id    

  inner join sys.objects so  

  on o.id = so.object_id  

     where o.name=@pvcParentTableName   

  and so.schema_id = schema_id(@pvcParentSchemaName)   

     and c.name=@pvcParentKey    

     )    

   BEGIN     

      Print '  Error Msg - Specified Table.Column2 ('+@pvcParentTableName+'.'+@pvcParentKey+') does not exist.  No action performed.'    

      goto DoneProcessing    

   END    

END --Single-column Key    

ELSE BEGIN --Multi-column Key    

   DECLARE @iPKeysInTable int    

   SELECT @iPKeysInTable = count(distinct t1.ColumnName)    

     from  sysobjects o    

     inner join syscolumns c    

     on o.id=c.id    

     inner join @tPKeys t1    

     on c.name = t1.ColumnName    

    

   if @iPKeysInTable <> @iPKeyCount    

   BEGIN     

      Print '  Error Msg - Specified Table.ColumnList ('+@pvcParentTableName+'.('+@pvcParentKey+') ) does not match.  One or more key does not exist.  No action performed.'    

      goto DoneProcessing    

   END    

END--Multi-column Key    

    

    

    

DECLARE @tbFKInfo  TABLE         

 (          

  ForeignKeyName varchar(100) NOT NULL,          

  FKTableName varchar(100) NOT NULL,     

 FKSchemaName varchar(100) NOT NULL,  

  FKCols varchar(800),    

  PKTableName varchar(100) NOT NULL,    

 PKSchemaname varchar(100) NOT NULL,   

  PKCols varchar(800)  ,  

 DELETE_REFERENTIAL_ACTION_DESC varchar(100),  

 UPDATE_REFERENTIAL_ACTION_DESC varchar(100)  

 )     

    

INSERT INTO @tbFKInfo SELECT * FROM dba_HelpFK(@pvcTableName, @pvcSchemaName)    

    

    

     

    

-------------------------------------------------    

-- Check if FK already exists (with same name)    

-------------------------------------------------    

IF Exists (select 1 from @tbFKInfo 
	where ForeignKeyName=@pvcConstraintName 
	AND FKTableName=@pvcTableName 
	AND FKCols=@pvcColumnName 
	AND PKTableName=@pvcParentTableName 
	AND PKCols=@pvcParentKey 
	AND FKSchemaName = @pvcSchemaName 
	AND PKSchemaname = @pvcParentSchemaName )     

BEGIN     
	--check to see if cascading has changed
	if exists (select 1 from @tbFKInfo 
	where ForeignKeyName=@pvcConstraintName 
	AND FKTableName=@pvcTableName 
	AND FKCols=@pvcColumnName 
	AND PKTableName=@pvcParentTableName 
	AND PKCols=@pvcParentKey 
	AND FKSchemaName = @pvcSchemaName 
	AND PKSchemaname = @pvcParentSchemaName 
	AND (
		CASE DELETE_REFERENTIAL_ACTION_DESC WHEN 'NO_ACTION' THEN 'NO ACTION' ELSE DELETE_REFERENTIAL_ACTION_DESC END <> @pvcDeleteAction
		OR CASE UPDATE_REFERENTIAL_ACTION_DESC WHEN 'NO_ACTION' THEN 'NO ACTION' ELSE UPDATE_REFERENTIAL_ACTION_DESC END <> @pvcUpdateAction
		)
	)
	begin
		PRINT '  Specified Foreign Key ('+@pvcTableName+'.'+@pvcConstraintName +') has different cascading options.  Dropping and recreating constraint.'
		SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' +@pvcTableName+' DROP CONSTRAINT '+@pvcConstraintName
		PRINT '    '+@SQL    
		EXEC(@SQL) 
		SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' +@pvcTableName+' ADD CONSTRAINT '+@pvcConstraintName+' FOREIGN KEY ('+ quotename(@pvcColumnName) +') REFERENCES '+ @pvcParentSchemaName + '.' + @pvcParentTableName+' ('+@pvcParentKey+')'    
		SELECT @SQL = @SQL + ' ON DELETE ' + @pvcDeleteAction   
		SELECT @SQL = @SQL + ' ON UPDATE ' + @pvcUpdateAction  
		PRINT '  Foreign Key will be created with the following command:'    
		PRINT '    '+@SQL    
		EXEC(@SQL)    
		
		goto DoneProcessing  
     
	end
	ELSE
	BEGIN

	   PRINT '  Specified Foreign Key ('+@pvcTableName+'.'+@pvcConstraintName +') with same attributes already exists.  No action taken.'     

	   goto DoneProcessing    

	    

	END    
END
 

    

    

---------------------------------------------------    

-- Check if FK already exists (with different name)    

---------------------------------------------------    

SELECT @vcConstraintName=NULL    

SELECT @vcConstraintName=ForeignKeyName    

  from @tbFKInfo     

  where FKTableName=@pvcTableName     

AND FKSchemaName = @pvcSchemaName  

  AND FKCols=@pvcColumnName     

  AND PKTableName=@pvcParentTableName     

AND PKSchemaName = @pvcParentSchemaName  

  AND PKCols=@pvcParentKey     

    

IF @vcConstraintName IS NOT NULL    

BEGIN    

   SET @SQL='exec sp_rename '''+@vcConstraintName+''','''+@pvcConstraintName+''''    

   PRINT '  Identical Foreign Key already exists but with different name ('+@vcConstraintName+').'    

   PRINT '  Foreign Key will be renamed with the following command:'    

   PRINT '    '+@SQL    

    

   EXEC(@SQL)    

    

   goto DoneProcessing    

END    

    

    

    

---------------------------------------------------    

-- Check if a FK with specified name already exists    

---------------------------------------------------    

SELECT @vcConstraintName=NULL    

SELECT @vcConstraintName=ForeignKeyName    

  from @tbFKInfo     

  where ForeignKeyName=@pvcConstraintName    

    

IF @vcConstraintName IS NOT NULL    

BEGIN    

   SET @SQL='exec sp_rename '''+@vcConstraintName+''','''+@vcConstraintName+'_tmpRename'''    

   PRINT '  Specified Foreign Key name already exists, Renaming with the following command:'    

   PRINT '    '+@SQL    

    

   EXEC(@SQL)    

    

END    

    

    

    

-------------------------    

-- Create the Foreign Key    

-------------------------    

  

--first check for referential actions  


declare @delete_ref_a_desc varchar(100)  

 ,@update_ref_a_desc varchar(100)  

  

SELECT @delete_ref_a_desc=delete_referential_action_desc  

 ,  @update_ref_a_desc=update_referential_action_desc  

from @tbFKInfo     

where ForeignKeyName=@pvcConstraintName    

  

SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' +@pvcTableName+' ADD CONSTRAINT '+@pvcConstraintName+' FOREIGN KEY ('+ quotename(@pvcColumnName) +') REFERENCES '+ @pvcParentSchemaName + '.' + @pvcParentTableName+' ('+@pvcParentKey+')'    

SELECT @SQL = @SQL + ' ON DELETE ' + @pvcDeleteAction   

SELECT @SQL = @SQL + ' ON UPDATE ' + @pvcUpdateAction  

PRINT '  Foreign Key will be created with the following command:'    

PRINT '    '+@SQL    

EXEC(@SQL)    

PRINT 'Done call to DBA_CreateForeignKey for ' + @pvcConstraintName  

Print '-------------------------------------------------------------------------------'   

    

DoneProcessing:    

    
