if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[DBA_CreateIndex]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[DBA_CreateIndex]
GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE DBA_CreateIndex 
(@pvcTableName varchar(1000)
,@pvcSchemaName varchar(100)
,@pvcIndexName varchar(2000)
,@pvcIndexKey varchar(2000)
,@pvcIndexIsClustered char(1)
,@pvcIndexIsUnique char(1)
,@pvnFillFactor TINYINT = 90
,@FileGroup varchar(200) = 'IDX')
AS
---------------------------------------------------------------------------------------
-- Stored Procedure to Create (or Alter) Index.
-- If Identical Index exists with same name - no action taken.
-- If Identical Index exists with different name - existing index is renamed.
-- If Index on same key exists but with differing attributes (Uniqueness or Clustering) 
--    existing dropped and new one created.
-- If New index is Clustering
--    If Clustering index exists - recreate it as NonClustered (including PKs)
-- We do not change filegroup assignments.  We specify where we want the index created initially
--    but after that we honor wherever another DBA may have decided to locate the index.  

---------------------------------------------------------------------------------------
SET NOCOUNT ON
DECLARE @vcIndexName varchar(1000)
DECLARE @vcIndexKey varchar(2000)
DECLARE @SQL varchar(1000)
DECLARE @SQL2 varchar(1000)
DECLARE @vcClustIndexName varchar(1000)
DECLARE @vcClustIndexKey varchar(2000)
DECLARE @vcUniqueClause varchar(2000)
DECLARE @vcClusteredClause varchar(2000)
DECLARE @cIsUnique char(1)

-- Remove "junk" from parameters
select @pvcTableName=replace(@pvcTableName,' ','')
select @pvcTableName=replace(@pvcTableName,'[','')
select @pvcTableName=replace(@pvcTableName,'] ','')
select @pvcTableName=replace(@pvcTableName,'dbo.','')

select @pvcIndexName=replace(@pvcIndexName,' ','')
select @pvcIndexName=replace(@pvcIndexName,'[','')
select @pvcIndexName=replace(@pvcIndexName,'] ','')
select @pvcIndexName=replace(@pvcIndexName,'dbo.','')

--select @pvcIndexKey=replace(@pvcIndexKey,'(-)','')
select @pvcIndexKey=replace(@pvcIndexKey,' ','')
--select @pvcIndexKey=replace(@pvcIndexKey,'(','')
--select @pvcIndexKey=replace(@pvcIndexKey,')','')
--select @pvcIndexKey=replace(@pvcIndexKey,'[','')
--select @pvcIndexKey=replace(@pvcIndexKey,'] ','')
--select @pvcIndexKey=replace(@pvcIndexKey,'dbo.','')

IF @pvcIndexIsClustered='0' SET @pvcIndexIsClustered='N'
IF @pvcIndexIsClustered='1' SET @pvcIndexIsClustered='Y'

IF @pvcIndexIsUnique='0' SET @pvcIndexIsUnique='N'
IF @pvcIndexIsUnique='1' SET @pvcIndexIsUnique='Y'


Print '-------------------------------------------------------------------------------'
Print 'Calling DBA_CreateIndex with '+@pvcTableName+'.'+@pvcIndexName+'('+@pvcIndexKey+') Clust='+@pvcIndexIsClustered+' Uniq='+@pvcIndexIsUnique
Print '-------------------------------------------------------------------------------'


declare @tbIndexInfo  TABLE   
 (    
  IndexName	varchar(1000) NOT NULL,    
  StatusCode	int,    
  IsPrimaryKey	char(1),  
  IsClustered	char(1),  
  IsUnique	char(1),  
  IndexKeys	varchar(4000)
 )    

INSERT INTO @tbIndexInfo SELECT * FROM dba_helpindex(@pvcTableName,@pvcSchemaName)
--SELECT * FROM dba_helpindex('tblLibrary','dbo')

------------------------------------------------------
-- Check to see if the same exact Index Already Exists
------------------------------------------------------
IF Exists (select 1 from @tbIndexInfo where IndexName=@pvcIndexName and IndexKeys=@pvcIndexKey and IsClustered=@pvcIndexIsClustered and IsUnique=@pvcIndexIsUnique)
BEGIN
   PRINT '  Specified index '+@pvcTableName+'.'+@pvcIndexName+'('+@pvcIndexKey+') already exists.  No action taken.'
   goto EndOfScript
END

------------------------------------------------------------------
-- Check to see if the same index exists but with a different name
------------------------------------------------------------------
SET @vcIndexName = NULL
SELECT @vcIndexName=IndexName
  FROM @tbIndexInfo 
  where IndexName<>@pvcIndexName 
  and IndexKeys=@pvcIndexKey 
  and IsClustered=@pvcIndexIsClustered 
  and IsUnique=@pvcIndexIsUnique
IF @vcIndexName IS NOT NULL
BEGIN
   Print '  Renaming existing index '+@vcIndexName+' to '+@pvcIndexName
   SELECT @SQL='sp_rename '''+@pvcSchemaname + '.' +@pvcTableName+'.'+@vcIndexName+''','''+@pvcIndexName+''',''INDEX'''

   SET @vcIndexKey=NULL
   SELECT @vcIndexKey=IndexKeys FROM @tbIndexInfo where IndexName=@pvcIndexName
   IF @vcIndexKey IS NOT NULL
   BEGIN
      SET @SQL2='drop index ' + @pvcSchemaName + '.' +@pvcTableName+'.'+@pvcIndexName
      Print '  Index with target name '+@pvcIndexName+' ('+@vcIndexKey+') already exists.  This index will be deleted'
      Print '  Drop Command: '+@SQL2
      EXEC (@SQL2)
   END
   exec (@SQL)
   goto EndOfScript
END

------------------------------------------------------------------
-- Check to see if index with same Key but Difference in either 
-- Clustering or Uniqueness
------------------------------------------------------------------
SET @vcIndexName = NULL
SELECT @vcIndexName=IndexName
  FROM @tbIndexInfo 
  where (     IndexKeys=@pvcIndexKey 
         and (  IsClustered<>@pvcIndexIsClustered 
             OR IsUnique<>@pvcIndexIsUnique
             )
        )
IF @vcIndexName IS NOT NULL
BEGIN
   SET @SQL='DROP INDEX ' + @pvcSchemaName + '.' +@pvcTableName+'.'+@vcIndexName
   Print '  Index '+@vcIndexName+' found with same key but different in either Clustering Or Uniqueness.  This index will be dropped.'  
   Print '  Drop Command: '+@SQL
   EXEC (@SQL)
END
ELSE BEGIN
   ---------------------------------------------------------------------
   -- Check to see if index with same Name exists but with Different Key
   ---------------------------------------------------------------------
   SET @vcIndexKey = NULL
   SELECT @vcIndexKey=IndexKeys FROM @tbIndexInfo where @pvcIndexName=IndexName AND @pvcIndexKey<>IndexKeys
   IF @vcIndexKey IS NOT NULL
   BEGIN
      SET @SQL='DROP INDEX '+ @pvcSchemaName + '.' + @pvcTableName+'.'+@pvcIndexName
      Print '  Index '+@pvcIndexName+' found with different key ('+@vcIndexKey+').  This index will be dropped.'
      Print '  Drop Command: '+@SQL
      DELETE FROM @tbIndexInfo where IndexName=@pvcIndexName
      EXEC (@SQL)
   END
END


IF @pvcIndexIsClustered='Y'
BEGIN
   --------------------------------------------------------------------------
   -- Check if Primary Key is Clustered
   --------------------------------------------------------------------------
   SET @vcClustIndexName = NULL
   Select @vcClustIndexName=IndexName, @vcClustIndexKey=IndexKeys
     from @tbIndexInfo
     where IsPrimaryKey='Y' and IsClustered='Y'

   IF @vcClustIndexName IS NOT NULL
   BEGIN
     Print '  Found Clustering Key on '+@pvcTableName+', Altering to Nonclustered'
     EXEC spCaptureAndDropFK @pvcTableName, @pvcSchemaName
     exec ('alter table ' + @pvcSchemaName + '.' +@pvcTableName+' drop constraint '+@vcClustIndexName)
     exec ('alter table ' + @pvcSchemaName + '.' +@pvcTableName+' add constraint '+@vcClustIndexName+' PRIMARY KEY NONCLUSTERED ('+@vcClustIndexKey+')')
     EXEC spRestoreFK @pvcTableName, @pvcSchemaName
   END
   ELSE BEGIN
      --------------------------------------------------------------------------
      -- Check if Clustered Index exists
      --------------------------------------------------------------------------
      SET @vcClustIndexName=NULL
      Select @vcClustIndexName=IndexName, @vcClustIndexKey=IndexKeys, @cIsUnique=IsUnique
        from @tbIndexInfo
        where IsClustered='Y'

      IF @vcClustIndexName IS NOT NULL and @vcClustIndexKey<>@pvcIndexKey
      BEGIN
         PRINT '  Found existing Clustered index '+@vcClustIndexName+' on ('+@vcClustIndexKey+'); Recreating as nonclustered.'
         Exec ('DROP INDEX ' + @pvcSchemaName + '.' +@pvcTableName+'.'+@vcClustIndexName)
      END
   END
END /* IF @pvcIndexIsClustered='Y' */


-----------------------
-- Create the new index
-----------------------
SELECT @vcUniqueClause = case when @pvcIndexIsUnique='Y' then 'UNIQUE' else '' end
SELECT @vcClusteredClause = case when @pvcIndexIsClustered='Y' then ' CLUSTERED' else '' end
SET @SQL='CREATE '+@vcUniqueClause+@vcClusteredClause+' INDEX '+@pvcIndexName+' ON '+ @pvcSchemaName + '.' + @pvcTableName+'('+@pvcIndexKey+')'

DECLARE @localFill TINYINT
IF @pvnFillFactor < 70
  set @localFill = 70
ELSE IF @pvnFillFactor > 100
  set @localFill = 90
ELSE IF @pvnFillFactor IS NULL
  set @localFill = 90
ELSE
  set @localFill = @pvnFillFactor
IF @localFill != @pvnFillFactor
  RAISERROR ( N'Changed FILLFACTOR from %d to %d', 18, 1, @pvnFillFactor, @localFill ) ;
SET @SQL=@SQL + ' WITH (FILLFACTOR = ' + CAST(@localFill AS VARCHAR) + ') ON '  + quotename(@FileGroup)

Print '  Creating Index with the following SQL:'
Print '    '+@SQL
Exec (@SQL)
  



EndOfScript:
Print '  Processing complete for '+@pvcTableName+'.'+@pvcIndexName+'('+@pvcIndexKey+').'
Print ' '

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

