if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[DBA_CreatePrimaryKey]') and OBJECTPROPERTY(id, N'IsProcedure') = 1)
drop procedure [dbo].[DBA_CreatePrimaryKey]
GO

SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO


CREATE PROCEDURE DBA_CreatePrimaryKey 
(@pvcTableName varchar(1000)
, @pvcSchemaName varchar(100)
, @pvcPrimaryKeyName varchar(2000)
, @pvcIndexKey varchar(2000)
, @pvcIndexIsClustered char(1)
, @FileGroup varchar(200) = 'PRIMARY')  
AS  
---------------------------------------------------------------------------------------  
-- Stored Procedure to Create (or Alter) Primary Key.  
-- If Identical PK exists with same name - no action taken.  
-- If Identical PK exists with different name - existing PK is renamed.  
-- If PK on same key exists but with differing attributes (Clustering)   
--    existing dropped and new one created.  
-- If New PK is Clustering  
--    If Clustering index already exists - recreate it as NonClustered  
-- We do not change filegroup assignments.  We specify where we want the index created initially
--    but after that we honor wherever another DBA may have decided to locate the index.  

SET NOCOUNT ON  
DECLARE @vcIndexName varchar(1000)  
DECLARE @vcIndexKey varchar(2000)  
DECLARE @SQL varchar(4000)  
DECLARE @vcClustIndexName varchar(1000)  
DECLARE @vcClustIndexKey varchar(2000)  
DECLARE @vcUniqueClause varchar(20)  
DECLARE @vcClusteredClause varchar(20)  
DECLARE @cIsUnique char(1)  
  
DECLARE @iNeedToRestoreFK int  
SET @iNeedToRestoreFK=0   
  
-- Remove "junk" from parameters  
select @pvcTableName=replace(@pvcTableName,' ','')  
select @pvcTableName=replace(@pvcTableName,'[','')  
select @pvcTableName=replace(@pvcTableName,'] ','')  
--select @pvcTableName=replace(@pvcTableName,'dbo.','')  
  
select @pvcPrimaryKeyName=replace(@pvcPrimaryKeyName,' ','')  
select @pvcPrimaryKeyName=replace(@pvcPrimaryKeyName,'[','')  
select @pvcPrimaryKeyName=replace(@pvcPrimaryKeyName,'] ','')  
--select @pvcPrimaryKeyName=replace(@pvcPrimaryKeyName,'dbo.','')  
  
select @pvcIndexKey=replace(@pvcIndexKey,'(-)','')  
select @pvcIndexKey=replace(@pvcIndexKey,' ','')  
select @pvcIndexKey=replace(@pvcIndexKey,'(','')  
select @pvcIndexKey=replace(@pvcIndexKey,')','')  
select @pvcIndexKey=replace(@pvcIndexKey,'[','')  
select @pvcIndexKey=replace(@pvcIndexKey,'] ','')  
--select @pvcIndexKey=replace(@pvcIndexKey,'dbo.','')  
  
IF @pvcIndexIsClustered='0' SET @pvcIndexIsClustered='N'  
IF @pvcIndexIsClustered='1' SET @pvcIndexIsClustered='Y'  
  
Print '-------------------------------------------------------------------------------'  
Print 'Calling DBA_CreatePrimaryKey with '+@pvcTableName+'.'+@pvcPrimaryKeyName+'('+@pvcIndexKey+') Clust='+@pvcIndexIsClustered  
Print '-------------------------------------------------------------------------------'  
  
  
declare @tbIndexInfo  TABLE     
 (      
  IndexName varchar(1000) NOT NULL,      
  StatusCode int,      
  IsPrimaryKey char(1),    
  IsClustered char(1),    
  IsUnique char(1),    
  IndexKeys varchar(4000)  
 )      
  
INSERT INTO @tbIndexInfo SELECT * FROM dba_helpindex(@pvcTableName, @pvcSchemaName)  
  
  
------------------------------------------------------  
-- Check to see if the same exact Index Already Exists  
------------------------------------------------------  
IF Exists (select 1 from @tbIndexInfo where IndexName=@pvcPrimaryKeyName and IndexKeys=@pvcIndexKey and IsClustered=@pvcIndexIsClustered and IsPrimaryKey='Y')  
BEGIN  
   PRINT '  Specified Primary Key '+@pvcTableName+'.'+@pvcPrimaryKeyName+'('+@pvcIndexKey+') already exists.  No action taken.'  
   goto EndOfScript  
END  
  
------------------------------------------------------------------  
-- Check to see if the same index exists but with a different name  
------------------------------------------------------------------  
SET @vcIndexName = NULL  
SELECT @vcIndexName=IndexName  
  FROM @tbIndexInfo   
  where IndexName<>@pvcPrimaryKeyName   
  and IndexKeys=@pvcIndexKey   
  and IsClustered=@pvcIndexIsClustered   
IF @vcIndexName IS NOT NULL  
BEGIN  
   Print '  Renaming existing Primary Key '+@vcIndexName+' to '+@pvcPrimaryKeyName  
   SELECT @SQL='sp_rename '''+ @pvcSchemaName + '.' + @pvcTableName+'.'+@vcIndexName+''','''+@pvcPrimaryKeyName+''''     
	PRINT '  SQL Executed:'+@SQL  
   exec (@SQL)  
   goto EndOfScript  
END  
  
------------------------------------------------------------------  
-- Check to see if index with same Key but Difference in Clustering  
------------------------------------------------------------------  
SET @vcIndexName = NULL  
SELECT @vcIndexName=IndexName  
  FROM @tbIndexInfo   
  where IndexKeys=@pvcIndexKey   
  and IsClustered<>@pvcIndexIsClustered   
  AND IsPrimaryKey='Y'  
IF @vcIndexName IS NOT NULL  
BEGIN  
   Print '  Primary Key '+@vcIndexName+' found with same key but different in Clustering.'  
   Print '  The Primary Key will be dropped and redefined.'  
   Print ' '  
   EXEC spCaptureAndDropFK @pvcTableName  , @pvcSchemaName
   Print ' '  
   SET @SQL='alter table '+ @pvcSchemaName + '.' + @pvcTableName+' drop constraint '+@vcIndexName  
   PRINT '  Drop Constraint Command: '+@SQL  
   exec (@SQL)  
   Print ' '  
   SET @iNeedToRestoreFK=1   

   DELETE FROM @tbIndexInfo
   INSERT INTO @tbIndexInfo SELECT * FROM dba_helpindex(@pvcTableName, @pvcSchemaName)  
END  
  
------------------------------------------------------------------  
-- Check to see if the Key has changed:  
------------------------------------------------------------------  
SET @vcIndexName = NULL  
SELECT @vcIndexName=IndexName  
     , @vcIndexKey=IndexKeys  
  FROM @tbIndexInfo   
  where IndexKeys<>@pvcIndexKey   
  AND IsPrimaryKey='Y'  
IF @vcIndexName IS NOT NULL  
BEGIN  
   Print '  Primary Key '+@vcIndexName+' found with different key ('+@vcIndexKey+').'  
   Print '  The Primary Key will be dropped and redefined.'  
   Print ' '  
   EXEC spCaptureAndDropFK @pvcTableName  , @pvcSchemaName
   Print ' '  
   SET @SQL='alter table '+ @pvcSchemaName + '.' + @pvcTableName+' drop constraint '+@vcIndexName  
   PRINT '  Drop Constraint Command: '+@SQL  
   exec (@SQL)  
   Print ' '  
   EXEC ('DROP TABLE tempport_'+@pvcTableName+'_FK')  

   DELETE FROM @tbIndexInfo
   INSERT INTO @tbIndexInfo SELECT * FROM dba_helpindex(@pvcTableName, @pvcSchemaName)  
END  
  
IF @pvcIndexIsClustered='Y'  
BEGIN  
  
   --------------------------------------------------------------------------  
   -- Check if Clustered Index exists  
   --------------------------------------------------------------------------  
   SET @vcClustIndexName=NULL  
   Select @vcClustIndexName=IndexName, @vcClustIndexKey=IndexKeys, @cIsUnique=IsUnique  
     from @tbIndexInfo  
     where IsClustered='Y'  
  
   IF @vcClustIndexName IS NOT NULL and @vcClustIndexKey<>@pvcIndexKey  
   BEGIN  
      PRINT '  Found existing Clustered index '+@vcClustIndexName+' on ('+@vcClustIndexKey+'); Recreating as nonclustered.'  
      SET @SQL='DROP INDEX ' + @pvcSchemaName + '.' +@pvcTableName+'.'+@vcClustIndexName  
      PRINT '  DROP Command: '+@SQL  
      Exec (@SQL)  
      IF @cIsUnique='Y' SET @vcUniqueClause='UNIQUE' ELSE SET @vcUniqueClause=''  
      SET @SQL='CREATE '+@vcUniqueClause+' INDEX '+@vcClustIndexName+' ON '+ @pvcSchemaName + '.' + @pvcTableName+'('+@vcClustIndexKey+')'  
      PRINT '  Re-Create Command: '+@SQL  
      Exec (@SQL)  
      PRINT ' '  

      DELETE FROM @tbIndexInfo
      INSERT INTO @tbIndexInfo SELECT * FROM dba_helpindex(@pvcTableName, @pvcSchemaName)  
   END  
END /* IF @pvcIndexIsClustered='Y' */  


------------------------------------------------------------------  
-- Check if indexs exists with same key
------------------------------------------------------------------ 
SET @vcIndexName = NULL  
SELECT @vcIndexName=IndexName  
  FROM @tbIndexInfo   
  where IndexKeys=replace(@pvcIndexKey,' ','')
  AND IsPrimaryKey='N'  
IF @vcIndexName IS NOT NULL  
BEGIN  
      PRINT '  Found existing duplicate index '+@vcIndexName+' with same key ('+@pvcIndexKey+'); Dropping.'
      SET @SQL='DROP INDEX '+ @pvcSchemaName + '.' + @pvcTableName+'.'+@vcIndexName  
      PRINT '  DROP Command: '+@SQL  
      Exec (@SQL)  
      PRINT ' '  
      GOTO CreatePK
END



------------------------------------------------------------------  
-- Check if indexs exists with same name
------------------------------------------------------------------ 
SET @vcIndexName = NULL  
SELECT @vcIndexName=IndexName  
  FROM @tbIndexInfo   
  where IndexName=@pvcPrimaryKeyName
  AND IsPrimaryKey='N'  
IF @vcIndexName IS NOT NULL  
BEGIN  
      PRINT '  Found existing index named '+@vcIndexName+'.  This index will be dropped.'
      SET @SQL='DROP INDEX '+ @pvcSchemaName + '.' + @pvcTableName+'.'+@vcIndexName  
      PRINT '  DROP Command: '+@SQL  
      Exec (@SQL)  
      PRINT ' '  
END



CreatePK:  
SELECT @vcClusteredClause = case when @pvcIndexIsClustered='Y' then ' CLUSTERED' else ' NONCLUSTERED' end  
SET @SQL='ALTER TABLE '+ @pvcSchemaName + '.' + @pvcTableName+' ADD CONSTRAINT '+@pvcPrimaryKeyName+' PRIMARY KEY '+@vcClusteredClause+' ('+@pvcIndexKey+') ON ' + QUOTENAME(@FileGroup)  
Print '  Creating Primary Key with the following SQL:'  
Print '    '+@SQL  
Print ' '  
  
Exec (@SQL)  
  
IF @iNeedToRestoreFK=1  
   EXEC spRestoreFK @pvcTableName  ,@pvcSchemaName
  
  
  
EndOfScript:  
Print '  Processing complete for '+@pvcTableName+'.'+@pvcPrimaryKeyName+'('+@pvcIndexKey+').'  
Print ' '  
  

GO
SET QUOTED_IDENTIFIER OFF 
GO
SET ANSI_NULLS ON 
GO

