if exists (select * from sysobjects where id = object_id('DBA_PortingAddorAlterColumn'))
begin
	drop proc DBA_PortingAddorAlterColumn
end
go

CREATE  Procedure DBA_PortingAddorAlterColumn    
( 
@pvcTableName		Varchar(100),    
@pvcSchemaName		Varchar(100),   
@pvcColumnName		Varchar(100),     
@pvcDataTypeText	varchar(100),		--INT, VARCHAR,DECIMAL,IMAGE,DATETIME,SMALLDATETIME,SMALLINT,TINYINT,MONEY,REAL
@pvcLength			varchar(100)=NULL,	--used for varchar/char cols only.  varchar(100) would be '100', varchar(max) would be 'MAX'
@pvcPrecision		varchar(100)=NULL,	--used for decimal and numeric only.  numeric(14,4) would be '14'
@pvcScale		varchar(100)=NULL,		--used for decimal and numeric only.  numeric(14,4) would be '4'
@pvcNullable Varchar(8)					--should be either NULL or NOT NULL    
,@DeleteCol	bit = 0						--should we just delete the col?  
)
AS    
---------------------------------------------------------------------------------------  
-- Stored Procedure to Create/Alter a column on a table 
-- Required Parms: TableName, SchemaName, ColumnName, DataType, Nullability 
-- Optional Parms: None  
--		If @DeleteCol is passed then delete the column
--		If identical col exists (same name, datatype, nullability) - no action taken
--		If same col name exists with different properties (nullability or datatype) then alter the col to match properties
---------------------------------------------------------------------------------------  
/*
test cases
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'varchar',
								@pvcLength = '100',
								@pvcPrecision = NULL,
								@pvcScale = NULL,
								@pvcNullable = 'NULL',
								@DeleteCol =  0								--table doesnt exist
drop table mxw.dave
create table mxw.dave (Column_B int NULL, COLUMN_C varchar(max) null)
sp_help 'mxw.dave'
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'varchar',
								@pvcLength = '100',
								@pvcPrecision = NULL,
								@pvcScale = NULL,
								@pvcNullable = 'NULL',
								@DeleteCol =  0								--create a varchar(100)
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'varchar',
								@pvcLength = '100',
								@pvcPrecision = NULL,
								@pvcScale = NULL,
								@pvcNullable = 'NOT NULL',
								@DeleteCol =  0								--change it to NOT NULL
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'varchar',
								@pvcLength = 'MAX',
								@pvcPrecision = NULL,
								@pvcScale = NULL,
								@pvcNullable = 'NOT NULL',
								@DeleteCol =  0								--change it to varchar(max)
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'SMALLDATETIME',
								@pvcLength = NULL,
								@pvcPrecision = NULL,
								@pvcScale = NULL,
								@pvcNullable = 'NOT NULL',
								@DeleteCol =  0								--change it to SMALLDATETIME
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'decimal',
								@pvcLength = NULL,
								@pvcPrecision = '10',
								@pvcScale = '2',
								@pvcNullable = 'NOT NULL',
								@DeleteCol =  0								--change it to decimal(10,2)
EXEC DBA_PortingAddorAlterColumn @pvcTableName = 'DAVE' , 
								@pvcSchemaName = 'MXW',
								@pvcColumnName = 'COLUMN_A',
								@pvcDataTypeText = 'decimal',
								@pvcLength = NULL,
								@pvcPrecision = '10',
								@pvcScale = '2',
								@pvcNullable = 'NOT NULL',
								@DeleteCol =  1								--delete the column
								

*/
  
SET NOCOUNT ON  
Print '-------------------------------------------------------------------------------'  
Print 'Calling DBA_PortingAddorAlterColumn with '+@pvcTableName+'.('+@pvcColumnName+')'  
 
declare @SQL varchar(1000)  , @col_string varchar(2000)
--------------------------------------  
-- Check Parms  
--------------------------------------  
  
--ensure table exists
if not exists (select * from sys.objects where name = @pvcTableName and schema_id = schema_id(@pvcSchemaName))
   BEGIN   
      Print '  Specified Table '+@pvcSchemaName+'.'+@pvcTableName+') does not exist.  No action performed.'  
      goto DoneProcessing  
   END  

--build datatype string for ADD COLUMN and ALTER COLUMN commands
IF (@pvcDataTypeText = 'INT'
	OR @pvcDataTypeText = 'SMALLINT'
	OR @pvcDataTypeText = 'BIGINT'
	OR @pvcDataTypeText = 'TINYINT'
	OR @pvcDataTypeText = 'MONEY'
	OR @pvcDataTypeText = 'REAL'
	OR @pvcDataTypeText = 'IMAGE'
	OR @pvcDataTypeText = 'TEXT'
	OR @pvcDataTypeText = 'DATETIME'
	OR @pvcDataTypeText = 'SMALLDATETIME'
	OR @pvcDataTypeText = 'UNIQUEIDENTIFIER'
	OR @pvcDataTypeText = 'BIT'
	OR @pvcDataTypeText = 'XML'
	)
BEGIN
	SELECT @col_string = @pvcDataTypeText
END
ELSE IF (@pvcDataTypeText = 'VARCHAR'
		OR @pvcDataTypeText = 'NVARCHAR'
		OR @pvcDataTypeText = 'CHAR'
		OR @pvcDataTypeText = 'NCHAR'
		OR @pvcDataTypeText = 'VARBINARY'
		)
BEGIN
	SELECT @col_string = @pvcDataTypeText + '(' + @pvcLength + ')'
	SELECT @pvcLength = CASE WHEN @pvcLength = 'MAX' THEN '-1' END
END
ELSE IF (@pvcDataTypeText = 'DECIMAL'
		OR @pvcDataTypeText = 'NUMERIC'
		)
BEGIN
	SELECT @col_string = @pvcDataTypeText + '(' + @pvcPrecision + ',' + @pvcScale + ')'
END	
ELSE IF (@pvcDataTypeText = 'TEXT')
BEGIN
	Print '  Error Msg : TEXT datatype is not supported in future SQL releases, use varchar(max).' 
	goto DoneProcessing 
END
ELSE
BEGIN
	Print '  Error Msg : Datatype : ' +  @pvcDataTypeText + ' is not supported by DBA_PortingAddorAlterColumn, contact DBA.'
	goto DoneProcessing
END



--check to see if col exists
if exists (	select c.* 
			from sys.columns c
			join sys.objects o on c.object_id = o.object_id
			where c.name = @pvcColumnName
			and o.name = @pvcTableName and o.schema_id = schema_id(@pvcSchemaName)
			)
			
BEGIN	--col exists
	if @DeleteCol = 1
	BEGIN	--deleting col
		SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' + @pvcTableName + ' DROP COLUMN ' + @pvcColumnName
		Print '  Running the Following SQL to drop column: ' + @SQL
		EXEC (@SQL)
		goto DoneProcessing 
	END		--deleting col
	ELSE
	BEGIN	--not deleting col
		--alter the column using the properties passed
		--check if identical col exists, do nothing
		if exists	(
						SELECT * 
						FROM sys.objects o
						JOIN sys.columns c on o.object_id = c.object_id
						JOIN sys.types t ON c.system_type_id = t.system_type_id
						WHERE o.name = @pvcTableName
						AND o.schema_id = schema_id(@pvcSchemaName)
						AND c.name = @pvcColumnName
						AND c.is_nullable = CASE WHEN @pvcNullable = 'NULL' THEN 1 ELSE 0 END
						AND t.name = @pvcDataTypeText
						AND (--if varchar then compare max_length...
							(c.max_length = @pvcLength OR @pvcLength IS NULL)
							OR ((c.precision = @pvcPrecision AND c.scale = @pvcScale) OR (@pvcPrecision IS NULL AND @pvcScale IS NULL))
							)
					)
		BEGIN
			Print '  Specified col requested to be altered already exists with identical properties. No action performed.' 
		END
		ELSE
		BEGIN
			SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' + @pvcTableName + ' ALTER COLUMN ' + @pvcColumnName + ' ' + @col_string + ' ' + @pvcNullable
			Print '  Running the Following SQL to ALTER column: ' + @SQL
			EXEC (@SQL)
			goto DoneProcessing
		END
	END		--not deleting col

END		--col exists
ELSE
BEGIN	--col does not exist
	if @DeleteCol = 1
	BEGIN	--deleting col
		Print '  Specified col requested to be deleted, but does not exist. No action performed.'  
		goto DoneProcessing
	END		--deleting col
	ELSE
	BEGIN	--not deleting col
		--add the column using properties passed
		SELECT @SQL = 'ALTER TABLE ' + @pvcSchemaName + '.' + @pvcTableName + ' ADD ' + @pvcColumnName
		SELECT @SQL = @SQL + ' ' + @col_string + ' ' + @pvcNullable
		Print '  Running the Following SQL to ADD column: ' + @SQL
		EXEC (@SQL)
		goto DoneProcessing	
	END		--not deleting col
END		--col does not exist


 
DoneProcessing:  
PRINT 'Done call to DBA_PortingAddorAlterColumn with '+@pvcTableName+'.('+@pvcColumnName+')'  
Print '-------------------------------------------------------------------------------' 
  
  
GO