------------------------------------------------------------------
--
--  File: spCaptureAndDropFK_prc.sql
--
--
--  Purpose:  This procedure is invoked during the porting process to 
--            capture information on and drop Foreign Keys for a particular
--            table.  During the porting process when a table needs to be 
--            dropped and recreated, all Foreign Keys referencing the table
--            must first be dropped.  This procedure captures all Foreign 
--            Key information on the table and saves it in a temporary 
--            table.  The Foreign Keys are then temporarily dropped.  
--            Control is then passed back to the porting scripts where the 
--            table can be safely dropped and recreated.  After the table 
--            is recreated, stored procedure spRestoreFK can be
--            executed to replace all the Foreign Keys.
--
--
--
IF EXISTS(SELECT 1
             FROM dbo.sysobjects
             WHERE id = OBJECT_ID(N'dbo.spCaptureAndDropFK')
             AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
   DROP PROC spCaptureAndDropFK
   PRINT 'Existing copy of spCaptureAndDropFK dropped; creating new one.'
END
GO

CREATE PROC spCaptureAndDropFK 
@pvcTableName varchar(120)
,@pvcSchemaName varchar(120)
AS

SET NOCOUNT ON

DECLARE @vcTempTableName varchar(120)
DECLARE @vcParentTable varchar(120)
DECLARE @vcChildTable varchar(120)
DECLARE @vcFKName varchar(120)
DECLARE @vcAltStmt varchar(1000)
DECLARE @vcThisKey   varchar(128)

DECLARE @iColno      int
DECLARE @iCount      int
DECLARE @iCnstID     int
DECLARE @iNumKeys    int
DECLARE @iRKeyid     int
DECLARE @iFKeyid     int

DECLARE  @iFKey1  int ,@iFKey2  int ,@iFKey3  int ,@iFKey4  int ,@iFKey5  int
        ,@iFKey6  int ,@iFKey7  int ,@iFKey8  int ,@iFKey9  int ,@iFKey10 int
        ,@iFKey11 int ,@iFKey12 int ,@iFKey13 int ,@iFKey14 int ,@iFKey15 int
        ,@iFKey16 int

DECLARE  @iRKey1  int ,@iRKey2  int ,@iRKey3  int ,@iRKey4  int ,@iRKey5  int
        ,@iRKey6  int ,@iRKey7  int ,@iRKey8  int ,@iRKey9  int ,@iRKey10 int
        ,@iRKey11 int ,@iRKey12 int ,@iRKey13 int ,@iRKey14 int ,@iRKey15 int
        ,@iRKey16 int

-- Assign name to temporary table
select @vcTempTableName = 'tempport_'+@pvcTableName+'_FK'

PRINT 'Capturing Foreign Key definitions and dropping Foreign Keys for table '+@pvcTableName
PRINT 'Captured Foreign Key definitions will be saved in table: '+@vcTempTableName

-- Create temporary table
-- If it already exists - exit.
select @iCount = count(1) 
  from dbo.sysobjects 
 where name = @vcTempTableName 
   and OBJECTPROPERTY(id, N'IsUserTable') = 1

If @iCount = 0
Begin
   exec ('CREATE TABLE '+@vcTempTableName+' (ParentTable varchar(120) NOT NULL, ChildTable varchar(120) NOT NULL, FKName varchar(120) NOT NULL, SQLStmt varchar(1000) NOT NULL)')
End
Else Begin
   PRINT 'ERROR: Table '+@vcTempTableName+' Already exists.  No processing performed.'  
   PRINT '  Exiting stored procedure.'
   PRINT ' '
   RETURN 
End


DECLARE cur_ForeignKeys CURSOR FOR 
	SELECT sr.constid 
	FROM sysreferences sr
	JOIN sys.objects so
	ON sr.rkeyid = so.object_id
	where (object_name(sr.rkeyid) = @pvcTableName
        or object_name(sr.fkeyid) = @pvcTableName)
	AND so.schema_id = schema_id (@pvcSchemaName)
	

create table #spcnstfkeys
(  cnst_colid int NOT NULL)

create table #spcnstpkeys
(  cnst_colid int NOT NULL)

open cur_ForeignKeys

fetch cur_ForeignKeys into @iCnstID
 while @@fetch_status >= 0
begin
   select @iFKeyid = fkeyid, @iRKeyid = rkeyid,
          @iFKey1=fkey1,     @iFKey2=fkey2,     @iFKey3=fkey3,
          @iFKey4=fkey4,     @iFKey5=fkey5,     @iFKey6=fkey6,
          @iFKey7=fkey7,     @iFKey8=fkey8,     @iFKey9=fkey9,
          @iFKey10=fkey10,   @iFKey11=fkey11,
          @iFKey12=fkey12,   @iFKey13=fkey13,
          @iFKey14=fkey14,   @iFKey15=fkey15,
          @iFKey16=fkey16,
          @iRKey1=rkey1,     @iRKey2=rkey2,     @iRKey3=rkey3,
          @iRKey4=rkey4,     @iRKey5=rkey5,     @iRKey6=rkey6,
          @iRKey7=rkey7,     @iRKey8=rkey8,     @iRKey9=rkey9,
          @iRKey10=rkey10,   @iRKey11=rkey11,
          @iRKey12=rkey12,   @iRKey13=rkey13,
          @iRKey14=rkey14,   @iRKey15=rkey15,
          @iRKey16=rkey16
     from sysreferences
    where constid = @iCnstID

   insert into #spcnstfkeys values(@iFKey1)
   insert into #spcnstfkeys values(@iFKey2)
   insert into #spcnstfkeys values(@iFKey3)
   insert into #spcnstfkeys values(@iFKey4)
   insert into #spcnstfkeys values(@iFKey5)
   insert into #spcnstfkeys values(@iFKey6)
   insert into #spcnstfkeys values(@iFKey7)
   insert into #spcnstfkeys values(@iFKey8)
   insert into #spcnstfkeys values(@iFKey9)
   insert into #spcnstfkeys values(@iFKey10)
   insert into #spcnstfkeys values(@iFKey11)
   insert into #spcnstfkeys values(@iFKey12)
   insert into #spcnstfkeys values(@iFKey13)
   insert into #spcnstfkeys values(@iFKey14)
   insert into #spcnstfkeys values(@iFKey15)
   insert into #spcnstfkeys values(@iFKey16)

   delete from #spcnstfkeys where cnst_colid = 0

   -- Need a unique index so we can use a cursor.
   create unique index indf1 on #spcnstfkeys(cnst_colid)

   insert into #spcnstpkeys values(@iRKey1)
   insert into #spcnstpkeys values(@iRKey2)
   insert into #spcnstpkeys values(@iRKey3)
   insert into #spcnstpkeys values(@iRKey4)
   insert into #spcnstpkeys values(@iRKey5)
   insert into #spcnstpkeys values(@iRKey6)
   insert into #spcnstpkeys values(@iRKey7)
   insert into #spcnstpkeys values(@iRKey8)
   insert into #spcnstpkeys values(@iRKey9)
   insert into #spcnstpkeys values(@iRKey10)
   insert into #spcnstpkeys values(@iRKey11)
   insert into #spcnstpkeys values(@iRKey12)
   insert into #spcnstpkeys values(@iRKey13)
   insert into #spcnstpkeys values(@iRKey14)
   insert into #spcnstpkeys values(@iRKey15)
   insert into #spcnstpkeys values(@iRKey16)

   delete from #spcnstpkeys where cnst_colid = 0

   ---- Need a unique index so we can use a cursor.

   create unique index indp1 on #spcnstpkeys(cnst_colid)

   select @vcAltStmt = 'ALTER TABLE '+ @pvcSchemaName + '.' + object_name(@iFKeyid)+'  ADD CONSTRAINT '+object_name(@iCnstID)

   execute('declare fkeycols_curs cursor for
             select cnst_colid from #spcnstfkeys
                for read only')

   open fkeycols_curs

   fetch fkeycols_curs into @iColno

   select @iNumKeys=1
   while @@fetch_status >= 0
   begin
      select @vcThisKey = col_name(@iFKeyid, @iColno)
      if @iNumKeys = 1
         select @vcAltStmt = @vcAltStmt+ ' FOREIGN KEY ('+ RTRIM(@vcThisKey)
      else
         select @vcAltStmt = @vcAltStmt+ ', '+ RTRIM(@vcThisKey)
      select @iNumKeys=@iNumKeys+1
      fetch fkeycols_curs into @iColno
   end --while key fetch

   select @vcAltStmt = @vcAltStmt+ ')'

   deallocate fkeycols_curs

   select @vcAltStmt = @vcAltStmt+' REFERENCES '+ @pvcSchemaName + '.' + object_name(@iRKeyid)

   execute('declare rkeycols_curs cursor for
             select cnst_colid from #spcnstpkeys
                for read only')

   open rkeycols_curs

   fetch rkeycols_curs into @iColno

   select @iNumKeys=1
   while @@fetch_status >= 0
   begin
      select @vcThisKey = col_name(@iRKeyid, @iColno)
      if @iNumKeys = 1
         select @vcAltStmt = @vcAltStmt+' ('+ RTRIM(@vcThisKey)	
      else
         select @vcAltStmt = @vcAltStmt+', '+ RTRIM(@vcThisKey)

      select @iNumKeys=@iNumKeys+1

      fetch rkeycols_curs into @iColno
   end --loop

   select @vcAltStmt = @vcAltStmt+')'
               
   deallocate rkeycols_curs

   Select @vcParentTable = object_name(@iRKeyid)
   Select @vcChildTable = object_name(@iFKeyid)
   Select @vcFKName = object_name(@iCnstID)

   Print '   Saving FK information for Table: '+@vcChildTable+' FK: '+@vcFKName
   exec ('INSERT INTO '+@vcTempTableName+'  (ParentTable, ChildTable, FKName, SQLStmt) Values ('''+@vcParentTable+''','''+@vcChildTable+''','''+@vcFKName+''','''+@vcAltStmt+''')')
   Print '      Dropping constraint...'
   exec ('alter table ' + @pvcSchemaName + '.' +@vcChildTable+' drop constraint '+@vcFKName)

   truncate table #spcnstfkeys
   drop index #spcnstfkeys.indf1

   truncate table #spcnstpkeys
   drop index #spcnstpkeys.indp1

   fetch cur_ForeignKeys into @iCnstID

end

drop table #spcnstpkeys
drop table #spcnstfkeys

close cur_ForeignKeys
deallocate cur_ForeignKeys

PRINT ' '
SET NOCOUNT OFF

GO