
--
--  Purpose:  This procedure is invoked during the porting process to 
--            re-create Foreign Keys for a particular table.  These Foreign
--            Keys were captured and dropped earlier in the porting process
--            by stored procedure spCaptureAndDropFK.  
--
--

--
IF EXISTS(SELECT 1
             FROM dbo.sysobjects
             WHERE id = OBJECT_ID(N'dbo.spRestoreFK')
             AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
   DROP PROC spRestoreFK
   PRINT 'Existing copy of spRestoreFK dropped; creating new one.'
END
GO

CREATE PROC spRestoreFK 
@pvcTableName varchar(120)
,@pvcSchemaName varchar(120)
AS

SET NOCOUNT ON

DECLARE @vcTempTableName varchar(120)
DECLARE @vcParentTable varchar(120)
DECLARE @vcChildTable varchar(120)
DECLARE @vcFKName varchar(120)
DECLARE @vcAltStmt varchar(1000)

DECLARE @i           int
DECLARE @iCount      int

-- Assign name to temporary table
select @vcTempTableName = 'tempport_'+@pvcTableName+'_FK'

PRINT 'Restoring Foreign Key definitions that reference table '+@pvcTableName
PRINT 'Captured Foreign Key definitions being read from table: '+@vcTempTableName

-- Verify that temporary table exists
select @iCount = count(1) 
  from dbo.sysobjects 
 where name = @vcTempTableName 
   and OBJECTPROPERTY(id, N'IsUserTable') = 1

If @iCount = 0
Begin
   PRINT 'ERROR: Table '+@vcTempTableName+' Not Found.  No processing performed.'  
   PRINT '  Exiting stored procedure.'
   PRINT ' '
   RETURN 
End


exec ('DECLARE cur_ForeignKeys CURSOR FOR 
	SELECT ParentTable
             , ChildTable
             , FKName 
             , SQLStmt
	FROM '+@vcTempTableName)

open cur_ForeignKeys

fetch cur_ForeignKeys into @vcParentTable, @vcChildTable, @vcFKName, @vcAltStmt

while @@fetch_status >= 0
begin
   Print '   Restoring Foreign Key '+@vcFKName+' on table '+@vcChildTable+'...'
   EXEC(@vcAltStmt)
   fetch cur_ForeignKeys into @vcParentTable, @vcChildTable, @vcFKName, @vcAltStmt
end

close cur_ForeignKeys
deallocate cur_ForeignKeys
PRINT 'Dropping temporary table '+@vcTempTableName
EXEC('Drop Table '+@vcTempTableName)

PRINT ' '
SET NOCOUNT OFF

GO