IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_CreateLookupValue'))
     BEGIN
          DROP PROCEDURE dbo.hsp_CreateLookupValue
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_CreateLookupValue]
(
  @groupName    NVARCHAR(50)
, @name         NVARCHAR(50)
, @description  NVARCHAR(255)
, @abbreviation NVARCHAR(50)
, @seqNo        INT           = 0
, @addData      NVARCHAR(100) = NULL
, @isDefault    BIT           = 0
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @lvgOID INT, @lvOID INT

  SELECT
    @lvgOID = LookupValueGroupOID
  FROM
    LookupValueGroups
  WHERE
    [Name] = @groupName

  IF (ISNULL(@lvgOID, 0) > 0)
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM LookupValues WHERE LookupValueGroupOID = @lvgOID AND [Name] = @name)
    BEGIN

      -- Get the next LookupValueGroupOID to use
      EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'LookupValue', @nextOID = @lvOID OUTPUT

      INSERT [dbo].[LookupValues]
      (
        [LookupValueOID]
      , [Name]
      , [Description]
      , [Abbreviation]
      , [LookupValueGroupOID]
      , [SequenceNumber]
      , [AdditionalData]
      , [IsDefault]
      )
      VALUES
      (
        @lvOID
      , @name
      , @description
      , @abbreviation
      , @lvgOID
      , @seqNo
      , @addData
      , @isDefault
      )
    END
    ELSE
    BEGIN
      -- LookupValue already exists
      RAISERROR (N'LookupValue %s already exists under group %s, NOT able to add.'
                ,0 -- Severity
                ,1 -- State
                ,@name
                ,@groupName);
    END
  END
  ELSE
  BEGIN
    -- LookupValueGroup NOT found
    RAISERROR (N'LookupValueGroup %s NOT found, unable to add LookupValue %s.'
              ,0 -- Severity
              ,1 -- State
              ,@groupName
              ,@name);
  END
END
GO
