IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_DropTableIndexes'))
     BEGIN
          DROP PROCEDURE dbo.hsp_DropTableIndexes
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_DropTableIndexes]
(
  @tableName SYSNAME
)
AS
BEGIN
  DECLARE @indexName SYSNAME
  DECLARE @tableOID INT

  SELECT @tableOID = OBJECT_ID(@tableName)

  IF EXISTS (SELECT 1 FROM [sys].[objects] WHERE object_id = @tableOID AND type = 'U')
  BEGIN

    -- If the table has a PRIMARY KEY CONSTRAINT we can't use DROP INDEX - it will
    --   need to be done seperately
    IF EXISTS (SELECT 1 FROM [sys].[objects] WHERE parent_object_id = @tableOID AND type = 'PK')
    BEGIN
       SELECT
         @indexName = name
       FROM
         [sys].[objects]
       WHERE
         parent_object_id = @tableOID
         AND type = 'PK'

       -- ALTER TABLE command does NOT allow scalar variables so we need
       --   to work around that limitation
       PRINT 'Dropping existing primary key ' + @indexName + ' on ' + @tableName
       EXEC('ALTER TABLE ' + @tableName + ' DROP CONSTRAINT ' + @indexName)
    END

    -- Now that we've taken care of the PRIMARY KEY CONSTRAINT (if defined)
    --   loop through the remaining indexes and drop them.
    IF EXISTS (SELECT 1 FROM [sys].[indexes] WHERE object_id = @tableOID AND name IS NOT NULL)
    BEGIN
      DECLARE
        Table_Indexes
      CURSOR FOR
      SELECT
        name
      FROM
        [sys].[indexes]
      WHERE
        object_id = @tableOID
        AND name IS NOT NULL

      OPEN Table_Indexes
      FETCH NEXT FROM Table_Indexes INTO @indexName

      WHILE @@FETCH_STATUS = 0
      BEGIN
        PRINT 'Dropping existing index ' + @indexName + ' on ' + @tableName
        EXEC('DROP INDEX ' + @indexName + ' ON ' + @tableName)

        FETCH NEXT FROM Table_Indexes INTO @indexName
      END
 
      CLOSE Table_Indexes
      DEALLOCATE Table_Indexes
    END
  END
END
GO
