IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_GetPatientsResultByFilterTest'))
     BEGIN
          DROP PROCEDURE dbo.hsp_GetPatientsResultByFilterTest
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- =============================================
-- Author:		Abey Peter
-- Create date: 02/19/2009
-- Description:	Returns the patients details
--              Along with the Exams taken for the latest visit
--				Along with the results for each exams					
-- =============================================
CREATE PROCEDURE [hsp_GetPatientsResultByFilterTest]
	-- Add the parameters for the stored procedure here
	--@name nvarchar(50) = null,
	@in_patientID nvarchar(15) = null,
	@in_pfirstName nvarchar(50) = null,
	@in_pmiddleName nvarchar(50) = null,
	@in_pLastName nvarchar(50) = NULL,
	@in_sDOB nvarchar(50) = NULL,
	@in_eDOB nvarchar(50) = NULL,
	@in_admtClass nvarchar(3) = null,
	@in_location nvarchar(50) = null,
	@in_gender nvarchar(1) = null,
	@in_admtType nvarchar(3) = null,
	@in_stlstExamDT nvarchar(50) = null,
	@in_edlstExamDT nvarchar(50) = null,
	@in_mrn nvarchar(50) = null,
	@in_account nvarchar(50) = null,
	@in_crt_admtPhy_FirstName nvarchar(50) = null,
	@in_crt_admtPhy_MiddleName nvarchar(50) = null,
	@in_crt_admtPhy_LastName nvarchar(50) = null,
	@in_crt_attendPhy_FirstName nvarchar(50) = null,
	@in_crt_attendPhy_MiddleName nvarchar(50) = null,
	@in_crt_attendPhy_LastName nvarchar(50) = null
	

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.

SET NOCOUNT ON;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED

DECLARE @patientSql varchar(8000)
DECLARE @examSql varchar(5000)
DECLARE @resultsql varchar(5000)
DECLARE @providerId varchar(300)
SET @providerId = null

create table #tempPatient ([PatientOID] [int] NULL,
	[FirstName] [nvarchar](50) NULL,
	[MiddleName] [nvarchar](50) NULL,
	[LastName] [nvarchar](50) NULL,
	[MRN] [nvarchar](50) NULL,
	[DateOfBirth] [datetime] NULL,
	[AdmitClass] [nvarchar](3) NULL,
	[AdmittingPhysician] [nvarchar](100) NULL,
	[AttendingPhysician] [nvarchar](100) NULL,
	[Location] [nvarchar](50) NULL,
	[AccountNumber] [nvarchar](50) NULL,
	[Gender] [nvarchar](1) NULL,
	[AdmitType] [nvarchar](3) NULL,
	[Height] [nvarchar](25) NULL,
	[Weight] [nvarchar](25) NULL,
	[IsPregnant] [nchar](1) NULL,
	[LMP] [smalldatetime] NULL,
	[VisitOID] [int] null)
	
    SET @patientSql = 'INSERT INTO #tempPatient SELECT 
		P.PatientOID,
		P.FirstName,
		P.MiddleName,
		P.LastName,
		dbo.fn_GetMRNByPatientID(P.PatientOID,''M'') AS MRN,
		P.DateOfBirth,
		V.AdmitClass,
		dbo.fn_GetCurrentPhysicianName(P.PatientOID,''Admit'') AS AdmittingPhysician,
		dbo.fn_GetCurrentPhysicianName(P.PatientOID,''Attend'') AS AttendingPhysician,
		V.NurseStation AS Location,
		dbo.fn_GetMRNByPatientID(P.PatientOID,''A'') AS AccountNumber,
		P.Gender,
		V.AdmitType,P.Height,P.Weight,P.IsPregnant,P.LMP,V.VisitOID from dbo.Patients P
		left join(select A.* from PatientVisits A Inner join
		(select PatientOID, max(isnull (AdmitDT,''01/01/1900'')) Maxadmitdate from PatientVisits
		group by PatientOID) B
		On A.PatientOID = B.PatientOID and A.AdmitDT = B.Maxadmitdate
		) V
		on P.PatientOID = V.PatientOID '
	
		IF((@in_mrn IS NOT NULL) OR (@in_account IS NOT NULL))
			BEGIN
				SET @patientSql = @patientSql + ' LEFT JOIN PatientIdentifiers I ON
							   V.PatientOID = I.IdentifierOID WHERE 1=1 ' 
			END
		ELSE
			BEGIN
				SET @patientSql = @patientSql + ' LEFT JOIN PatientContacts C ON P.PatientOID = C.PatientOID WHERE 1 = 1 '
			END
 
		
	IF(@in_patientID is not null)
		BEGIN
			SET @patientSql = @patientSql + ' and P.PatientOID = ' + @in_patientID
		END	
	IF(@in_pfirstName is not null)
		BEGIN
			SET @patientSql = @patientSql + ' and P.FirstName LIKE ' + quotename(@in_pfirstName + '%','''') 
		END
	IF(@in_pmiddleName is not null)
		BEGIN
			SET @patientSql = @patientSql + ' and P.MiddleName LIKE ' + quotename(@in_pmiddleName + '%','''') 
		END
	IF(@in_pLastName is not null)
		BEGIN
			SET @patientSql = @patientSql + ' and P.LastName LIKE ' + quotename(@in_pLastName + '%','''') 
		END
	IF(@in_sDOB IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and P.DateOfBirth BETWEEN ' + quotename(@in_sDOB,'''') + ' AND ' + quotename(@in_eDOB,'''')
		END
	IF(@in_admtClass IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and V.AdmitClass = ' + quotename(@in_admtClass,'''')
		END
	IF(@in_location IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and V.NurseStation = ' + quotename(@in_location,'''')
		END 
	IF(@in_gender IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and P.Gender = ' + quotename(@in_gender,'''')
		END
	IF(@in_admtType IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and V.AdmitType = ' + quotename(@in_admtType,'''')
		END
	IF (@in_stlstExamDT IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and P.LastExamDT BETWEEN ' + quotename(@in_stlstExamDT,'''') + ' AND ' + quotename(@in_edlstExamDT,'''')
		END 
	IF(@in_mrn IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and I.ID = ' + quotename(@in_mrn,'''') + ' and I.IdentifierType = ''MRN'''
		END
	IF(@in_account IS NOT NULL)
		BEGIN
			SET @patientSql = @patientSql + ' and I.ID = ' + quotename(@in_account,'''') + ' and I.IdentifierType = ''ADM'''
		END
	IF(@in_crt_admtPhy_FirstName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AdmittingPhysician'
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.FirstName LIKE ' + quotename(@in_crt_admtPhy_FirstName + '%','''') 
		END
	IF(@in_crt_admtPhy_MiddleName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AdmittingPhysician'
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.MiddleName LIKE ' + quotename(@in_crt_admtPhy_MiddleName + '%','''') 
		END
	IF(@in_crt_admtPhy_LastName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AdmittingPhysician'	
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.LastName LIKE ' + quotename(@in_crt_admtPhy_LastName + '%','''') 
		END		
	IF(@in_crt_attendPhy_FirstName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AttendingPhysician'
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.FirstName LIKE ' + quotename(@in_crt_attendPhy_FirstName + '%','''') 
		END
	IF(@in_crt_attendPhy_MiddleName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AttendingPhysician'
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.MiddleName LIKE ' + quotename(@in_crt_attendPhy_MiddleName + '%','''') 
		END
	IF(@in_crt_attendPhy_LastName is not null)
		BEGIN
			select @providerId = ProviderRoleOID from dbo.ProviderRoles where
			Name = 'AttendingPhysician'	
			SET @patientSql = @patientSql + ' and C.ProviderRoleOID = ' + @providerId + ' and C.LastName LIKE ' + quotename(@in_crt_attendPhy_LastName + '%','''') 
		END	
		

		 
PRINT @patientSql 

 EXEC (@patientSql)
	 
	SELECT * FROM #tempPatient
	
	SELECT P.PatientOID,E.ExamOID,E.OrderNumber,E.Description,E.AccessionNumber,E.PerformedOnDT,
	dbo.fn_GetReferringPhysicianForExam(P.PatientOID) AS ReferringPhysician FROM
	dbo.PatientExams E,dbo.#tempPatient P where P.PatientOID = E.PatientOID and P.VisitOID = E.VisitOID


SELECT 
ExamResultMap.ExamOID, PatientResults.PatientOID, PatientResults.PatientResultOID, PatientResults.ResultStatus, PatientResults.ResultText 
FROM PatientResults
Inner join ExamResultMap
On PatientResults.PatientResultOID = ExamResultMap.ResultOID
Inner join PatientExams
On ExamResultMap.ExamOID = PatientExams.ExamOID
Inner Join #tempPatient P ON PatientExams.PatientOID = P.PatientOID
where PatientExams.VisitOID = P.VisitOID
 
UNION ALL

SELECT 
ExamResultMap.ExamOID, PatientResults.PatientOID, PatientResults.PatientResultOID, PatientResults.ResultStatus, PatientResults.ResultText 

FROM 
	(
	select A.*, B.PatientResultOID B_ResultOID
	from PatientResults A
	Inner Join (select PatientResultOID,PreviousResultOID from PatientResults where PreviousResultOID is not null) B
	On A.PatientResultOID = B.PreviousResultOID
	) PatientResults
Inner join ExamResultMap
On PatientResults.B_ResultOID = ExamResultMap.ResultOID
Inner join PatientExams
On ExamResultMap.ExamOID = PatientExams.ExamOID
Inner Join #tempPatient P ON PatientExams.PatientOID = P.PatientOID
where PatientExams.VisitOID = P.VisitOID

DROP TABLE #tempPatient
END
GO
