IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateField'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateField
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateField]
(
  @interfaceName               NVARCHAR(8)
, @fieldName                   NVARCHAR(50)
, @fieldDescription            NVARCHAR(200)
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID INT

  SELECT @interfaceOID = 0

  SELECT
    @interfaceOID = ID.InterfaceDefinitionOID
  FROM
    InterfaceDefinitions ID
  WHERE
    ID.InterfaceName    = @interfaceName

  IF (ISNULL(@interfaceOID, 0) > 0)
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM InterfaceFieldDefinitions IFD
                     WHERE IFD.InterfaceDefinitionOID = @interfaceOID AND IFD.FieldName = @fieldName)
    BEGIN

      INSERT InterfaceFieldDefinitions
      (
        [InterfaceDefinitionOID]
      , [FieldName]
      , [FieldDescription]
      )
      VALUES
      (
        @interfaceOID
      , @fieldName
      , @fieldDescription
      )
    END
    ELSE
    BEGIN
        -- Field definition already exists on the passed interface
        RAISERROR (N'%s field already defined on %s interface.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@fieldName
                  ,@interfaceName);
    END
  END
  ELSE
  BEGIN
    -- Interface definition NOT found
    RAISERROR (N'%s interface definition NOT found, unable to add %s field.'
              ,20 -- Severity
              ,1  -- State
              ,@interfaceName
              ,@fieldName); 
  END
END
GO
