IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateFieldAction'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateFieldAction
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateFieldAction]
(
  @interfaceName               NVARCHAR(8)
, @messageType                 NVARCHAR(3)
, @eventType                   NVARCHAR(3)
, @fieldName                   NVARCHAR(50)
, @fieldAction                 NVARCHAR(20)
, @actionParam1                NVARCHAR(12) = NULL
, @actionParam2                NVARCHAR(12) = NULL
, @actionParam3                NVARCHAR(12) = NULL
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID INT
  DECLARE @msgOID       INT

  SELECT @interfaceOID = 0, @msgOID = 0

  SELECT
    @interfaceOID = ID.InterfaceDefinitionOID
  , @msgOID       = IMD.InterfaceMessageDefinitionOID
  FROM
    InterfaceDefinitions ID
    INNER JOIN InterfaceMessageDefinitions IMD
      ON IMD.InterfaceDefinitionOID = ID.InterfaceDefinitionOID
  WHERE
    ID.InterfaceName    = @interfaceName
    AND IMD.MessageType = @messageType
    AND IMD.EventType   = @eventType

  IF (ISNULL(@interfaceOID, 0) > 0 AND ISNULL(@msgOID, 0) > 0)
  BEGIN
    IF EXISTS (SELECT 1 FROM InterfaceFieldDefinitions
                 WHERE InterfaceDefinitionOID = @interfaceOID AND FieldName = @fieldName)
    BEGIN
      IF NOT EXISTS (SELECT 1 FROM InterfaceFieldActions IFA
                       WHERE IFA.InterfaceMessageDefinitionOID = @msgOID
                         AND IFA.FieldName = @fieldName AND IFA.FieldAction = @fieldAction)
      BEGIN

        INSERT InterfaceFieldActions
        (
          [InterfaceMessageDefinitionOID]
        , [FieldName]
        , [FieldAction]
        , [ActionParam1]
        , [ActionParam2]
        , [ActionParam3]
        )
        VALUES
        (
          @msgOID
        , @fieldName
        , @fieldAction
        , @actionParam1
        , @actionParam2
        , @actionParam3
        )
      END
      ELSE
      BEGIN
          -- Field action already exists on the message
          RAISERROR (N'%s action already defined on %s field in %s interface message %s:%s.'
                    ,0 -- Severity
                    ,1 -- State
                    ,@fieldAction
                    ,@fieldName
                    ,@interfaceName
                    ,@messageType
                    ,@eventType); 
      END
    END
    ELSE
    BEGIN
      -- The field definition for this mapping NOT found
      RAISERROR (N'%s field NOT defined for %s interface message (%s:%s), unable to add %s field action.'
                ,20 -- Severity
                ,1  -- State
                ,@fieldName
                ,@interfaceName
                ,@messageType
                ,@eventType
                ,@fieldAction); 
    END
  END
  ELSE
  BEGIN
    -- Parent message NOT found for passed interface
    RAISERROR (N'Message (%s:%s) NOT defined for %s interface, unable to add %s field %s action.'
              ,20 -- Severity
              ,1  -- State
              ,@messageType
              ,@eventType
              ,@interfaceName
              ,@fieldName 
              ,@fieldAction); 
  END
END
GO
