IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateFieldMapping'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateFieldMapping
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateFieldMapping]
(
  @interfaceName               NVARCHAR(8)
, @messageType                 NVARCHAR(3)
, @eventType                   NVARCHAR(3)
, @segmentName                 NVARCHAR(3)
, @fieldName                   NVARCHAR(50)
, @fieldType                   NVARCHAR(10)
, @fieldSubType                NVARCHAR(10) = NULL
, @segmentLocation             NVARCHAR(2)  = N'1'
, @fieldPosition               INT          = 0
, @componentPosition           INT          = 1
, @descriptorValue             NVARCHAR(20) = NULL
, @descriptorFieldPosition     INT          = NULL
, @descriptorComponentPosition INT          = NULL
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID INT
  DECLARE @msgOID       INT

  SELECT @interfaceOID = 0, @msgOID = 0

  SELECT
    @interfaceOID = ID.InterfaceDefinitionOID
  , @msgOID       = IMD.InterfaceMessageDefinitionOID
  FROM
    InterfaceDefinitions ID
    INNER JOIN InterfaceMessageDefinitions IMD
      ON IMD.InterfaceDefinitionOID = ID.InterfaceDefinitionOID
  WHERE
    ID.InterfaceName    = @interfaceName
    AND IMD.MessageType = @messageType
    AND IMD.EventType   = @eventType

  IF (ISNULL(@interfaceOID, 0) > 0 AND ISNULL(@msgOID, 0) > 0)
  BEGIN
    IF EXISTS (SELECT 1 FROM InterfaceFieldDefinitions
                 WHERE InterfaceDefinitionOID = @interfaceOID AND FieldName = @fieldName)
    BEGIN
      IF NOT EXISTS (SELECT 1 FROM InterfaceFieldMappings IFM
                       WHERE IFM.InterfaceMessageDefinitionOID = @msgOID AND IFM.FieldName = @fieldName)
      BEGIN

        INSERT InterfaceFieldMappings
        (
          [InterfaceMessageDefinitionOID]
        , [FieldName]
        , [FieldType]
        , [FieldSubType]
        , [SegmentName]
        , [SegmentLocation]
        , [FieldPosition]
        , [ComponentPosition]
        , [DescriptorValue]
        , [DescriptorFieldPosition]
        , [DescriptorComponentPosition]
        )
        VALUES
        (
          @msgOID
        , @fieldName
        , @fieldType
        , @fieldSubType
        , @segmentName
        , @segmentLocation
        , @fieldPosition
        , @componentPosition
        , @descriptorValue
        , @descriptorFieldPosition
        , @descriptorComponentPosition
        )
      END
      ELSE
      BEGIN
          -- Field definition already exists on the message
          RAISERROR (N'%s field mapping already defined on %s interface message (%s:%s).'
                    ,0 -- Severity
                    ,1 -- State
                    ,@fieldName
                    ,@interfaceName
                    ,@messageType
                    ,@eventType); 
      END
    END
    ELSE
    BEGIN
      -- The field definition for this mapping NOT found
      RAISERROR (N'%s field NOT defined for %s interface message (%s:%s), unable to add field mapping.'
                ,20 -- Severity
                ,1  -- State
                ,@messageType
                ,@eventType
                ,@interfaceName
                ,@fieldName); 
    END
  END
  ELSE
  BEGIN
    -- Parent message NOT found for passed interface
    RAISERROR (N'Message (%s:%s) NOT defined for %s interface, unable to add %s field mapping.'
              ,20 -- Severity
              ,1  -- State
              ,@messageType
              ,@eventType
              ,@interfaceName
              ,@fieldName); 
  END
END
GO
