IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateFieldRelation'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateFieldRelation
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateFieldRelation]
(
  @interfaceName               NVARCHAR(8)
, @messageType                 NVARCHAR(3)
, @eventType                   NVARCHAR(3)
, @parentFieldName             NVARCHAR(50)
, @childFieldName              NVARCHAR(50)
, @childFieldOrder             INT
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID      INT
  DECLARE @msgOID            INT

  SELECT @interfaceOID = 0, @msgOID = 0

  SELECT
    @interfaceOID       = ID.InterfaceDefinitionOID
  , @msgOID             = IMD.InterfaceMessageDefinitionOID
  FROM
    InterfaceDefinitions ID
    INNER JOIN InterfaceMessageDefinitions IMD
      ON IMD.InterfaceDefinitionOID = ID.InterfaceDefinitionOID
    INNER JOIN InterfaceFieldMappings IFM
      ON IFM.InterfaceMessageDefinitionOID = IMD.InterfaceMessageDefinitionOID
  WHERE
    ID.InterfaceName    = @interfaceName
    AND IMD.MessageType = @messageType
    AND IMD.EventType   = @eventType
    AND IFM.FieldName   = @parentFieldName

  IF (ISNULL(@interfaceOID, 0) > 0 AND ISNULL(@msgOID, 0) > 0)
  BEGIN

    IF EXISTS (SELECT 1 FROM InterfaceFieldMappings IFM
                 WHERE IFM.InterfaceMessageDefinitionOID = @msgOID AND IFM.FieldName = @childFieldName)
    BEGIN

      IF NOT EXISTS (SELECT 1 FROM InterfaceParentChildFieldMap
                       WHERE InterfaceMessageDefinitionOID = @msgOID
                       AND ParentFieldName = @parentFieldName AND ChildFieldName = @childFieldName)
      BEGIN
        INSERT InterfaceParentChildFieldMap
        (
          InterfaceMessageDefinitionOID
        , ParentFieldName
        , ChildFieldName
        , ChildOrder
        )
        VALUES
        (
          @msgOID
        , @parentFieldName
        , @childFieldName
        , @childFieldOrder
        )
      END
      ELSE
      BEGIN
        -- Relation already exists
        RAISERROR (N'Relation already exists between %s and %s on interface %s message %s:%s.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@parentFieldName
                  ,@childFieldName
                  ,@interfaceName
                  ,@messageType
                  ,@eventType);
      END
    END
    ELSE
    BEGIN
      -- Child segment/message/field NOT found for passed interface
      RAISERROR (N'Message (%s:%s) or field (%s) NOT defined for %s interface, unable to create relation.'
                ,0 -- Severity
                ,1 -- State
                ,@messageType
                ,@eventType
                ,@childFieldName
                ,@interfaceName); 
    END
  END
  ELSE
  BEGIN
    -- Parent message/field NOT found for passed interface
    RAISERROR (N'Message (%s:%s) or field (%s) NOT defined for %s interface, unable to create relation.'
              ,0 -- Severity
              ,1 -- State
              ,@messageType
              ,@eventType
              ,@parentFieldName
              ,@interfaceName); 
  END
END
GO
