IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateMsg'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateMsg
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateMsg]
(
  @interfaceName               NVARCHAR(8)
, @messageType                 NVARCHAR(3)
, @eventType                   NVARCHAR(3)
, @messageDescription          NVARCHAR(200)
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID INT
  DECLARE @msgOID       INT

  SELECT @interfaceOID = 0, @msgOID = 0

  SELECT
    @interfaceOID = ID.InterfaceDefinitionOID
  FROM
    InterfaceDefinitions ID
  WHERE
    ID.InterfaceName = @interfaceName

  IF (ISNULL(@interfaceOID, 0) > 0)
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM InterfaceMessageDefinitions IMD
                     WHERE IMD.InterfaceDefinitionOID = @interfaceOID
                       AND IMD.MessageType = @messageType AND IMD.EventType = @eventType)
    BEGIN
      -- Get the next InterfaceMessageDefinitionOID to use
      EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'InterfaceMessageDefinition', @nextOID = @msgOID OUTPUT

      INSERT InterfaceMessageDefinitions
      (
        [InterfaceMessageDefinitionOID]
      , [MessageType]
      , [EventType]
      , [Description]
      , [InterfaceDefinitionOID]
      )
      VALUES
      (
        @msgOID
      , @messageType
      , @eventType
      , @messageDescription
      , @interfaceOID
      )
    END
    ELSE
    BEGIN
        -- Msg definition already exists on the passed interface
        RAISERROR (N'Message (%s:%s) already exists on interface (%s), NOT able to add.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@messageType
                  ,@eventType
                  ,@interfaceName);
    END
  END
  ELSE
  BEGIN
    -- Passed interface definition NOT found
    RAISERROR (N'Interface (%s) not found, NOT able to add message (%s:%s).'
              ,0 -- Severity
              ,1 -- State
              ,@interfaceName
              ,@messageType
              ,@eventType);
  END
END
GO
