IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceCreateMsgData'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceCreateMsgData
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceCreateMsgData]
(
  @interfaceName               NVARCHAR(8)
, @messageType                 NVARCHAR(3)
, @eventType                   NVARCHAR(3)
, @category                    NVARCHAR(50)
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceOID        INT
  DECLARE @msgOID              INT
  DECLARE @lookupValueGroupOID INT

  SELECT @interfaceOID = 0, @msgOID = 0, @lookupValueGroupOID = 0

  SELECT
    @interfaceOID = ID.InterfaceDefinitionOID
  , @msgOID       = IMD.InterfaceMessageDefinitionOID
  FROM
    InterfaceDefinitions ID
    LEFT OUTER JOIN InterfaceMessageDefinitions IMD
      ON IMD.InterfaceDefinitionOID = ID.InterfaceDefinitionOID
  WHERE
    ID.InterfaceName    = @interfaceName
    AND IMD.MessageType = @messageType
    AND IMD.EventType   = @eventType

  IF (ISNULL(@interfaceOID, 0) > 0 AND ISNULL(@msgOID, 0) > 0)
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM InterfaceMessageDataMap IMDM
                     INNER JOIN LookupValueGroups LVG ON LVG.LookupValueGroupOID = IMDM.LookupValueGroupOID
                     WHERE IMDM.InterfaceMessageDefinitionOID = @msgOID AND LVG.Name = @category)
    BEGIN

      SELECT
        @lookupValueGroupOID = LVG.LookupValueGroupOID
      FROM
        LookupValueGroups LVG
      WHERE
        LVG.Name = @category

      IF (ISNULL(@lookupValueGroupOID, 0) > 0)
      BEGIN
        INSERT InterfaceMessageDataMap
        (
          [InterfaceMessageDefinitionOID]
        , [LookupValueGroupOID]
        )
        VALUES
        (
          @msgOID
        , @lookupValueGroupOID
        )
      END
      ELSE
      BEGIN
        -- Msg data definition NOT found
        RAISERROR (N'Message data definition (%s) NOT found, unable to assign to interface (%s) message (%s:%s).'
                  ,0 -- Severity
                  ,1 -- State
                  ,@category
                  ,@interfaceName
                  ,@messageType
                  ,@eventType);
      END
    END
    ELSE
    BEGIN
        -- Msg definition already exists on the passed interface
        RAISERROR (N'Message data (%s) already assigned to interface (%s) message (%s:%s), NOT able to add.'
                  ,0 -- Severity
                  ,1 -- State
                  ,@category
                  ,@interfaceName
                  ,@messageType
                  ,@eventType);
    END
  END
  ELSE
  BEGIN
    -- Passed interface definition NOT found
    RAISERROR (N'Interface (%s) message (%s:%s) not found, NOT able to associate message data (%s).'
              ,0 -- Severity
              ,1 -- State
              ,@interfaceName
              ,@messageType
              ,@eventType
              ,@category);
  END
END
GO
