IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceGetMsgFieldsToProcess'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceGetMsgFieldsToProcess
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceGetMsgFieldsToProcess]
(
 @msgDefinitionOID INT
)
AS
BEGIN

  SET NOCOUNT ON

  DECLARE @interfaceDefinitionOID INT

  SELECT
    @interfaceDefinitionOID = IMD.InterfaceDefinitionOID
  FROM
    InterfaceMessageDefinitions IMD
  WHERE
    IMD.InterfaceMessageDefinitionOID = @msgDefinitionOID

  SELECT
     IFD.FieldName                   AS 'FieldName'
    ,IFM.FieldType                   AS 'FieldType'
    ,IFM.FieldSubType                AS 'FieldSubType'
    ,IFD.FieldDescription            AS 'Description'
    ,IFM.SegmentName                 AS 'ParentSegment'
    ,IFM.SegmentLocation             AS 'ParentSegmentLocation'
    ,IFM.FieldPosition               AS 'FieldPosition'
    ,IFM.ComponentPosition           AS 'ComponentPosition'
    ,IFM.DescriptorValue             AS 'DescriptorValue'
    ,IFM.DescriptorFieldPosition     AS 'DescriptorFieldPosition'
    ,IFM.DescriptorComponentPosition AS 'DescriptorComponentPosition'
  FROM
     InterfaceFieldMappings IFM
       INNER JOIN InterfaceFieldDefinitions IFD
         ON IFD.FieldName = IFM.FieldName
  WHERE
     IFM.InterfaceMessageDefinitionOID = @msgDefinitionOID
     AND IFD.InterfaceDefinitionOID = @interfaceDefinitionOID
  ORDER BY
     IFM.SegmentName, IFM.FieldPosition, IFM.ComponentPosition

  SELECT
     PARENT.FieldName  AS 'ParentFieldName'
    ,CHILD.FieldName   AS 'ChildFieldName'
    ,IPCFM.ChildOrder  AS 'ChildFieldOrder'
  FROM
     InterfaceParentChildFieldMap IPCFM
       INNER JOIN InterfaceFieldMappings PARENT
         ON PARENT.InterfaceMessageDefinitionOID = IPCFM.InterfaceMessageDefinitionOID
            AND PARENT.FieldName = IPCFM.ParentFieldName
       INNER JOIN InterfaceFieldMappings CHILD
         ON CHILD.InterfaceMessageDefinitionOID = IPCFM.InterfaceMessageDefinitionOID
            AND CHILD.FieldName = IPCFM.ChildFieldName
  WHERE
     IPCFM.InterfaceMessageDefinitionOID = @msgDefinitionOID
  ORDER BY
     PARENT.FieldName, IPCFM.ChildOrder

  SELECT
     IFA.FieldName     AS 'FieldName'
    ,IFA.FieldAction   AS 'Action'
    ,IFA.ActionParam1  AS 'Param1'
    ,IFA.ActionParam2  AS 'Param2'
    ,IFA.ActionParam3  AS 'Param3'
  FROM
     InterfaceFieldActions IFA
       INNER JOIN InterfaceFieldMappings IFM
         ON IFM.InterfaceMessageDefinitionOID = IFA.InterfaceMessageDefinitionOID
            AND IFM.FieldName = IFA.FieldName
  WHERE
     IFA.InterfaceMessageDefinitionOID = @msgDefinitionOID
  ORDER BY
     IFA.FieldName, IFA.FieldAction
END
GO
