IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceGetResultByExamData'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceGetResultByExamData
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceGetResultByExamData]
(
  @patientOID     INT
, @visitOID       INT
, @entity         NVARCHAR(12)
, @uniServiceCode NVARCHAR(12)
, @orderedForDT   SMALLDATETIME
, @orderNumber    NVARCHAR(20)
, @accItn         NVARCHAR(12)
)
AS
BEGIN

  SET NOCOUNT ON

  -- Variable Declarations
  DECLARE @examOID INT
  DECLARE @resultOID INT
  DECLARE @previousResultOID INT

  -- Initialize Variables
  SELECT @examOID = 0, @resultOID = 0, @previousResultOID = 0

  -- See if we can find the exam the sp call is for.  If not we'll raise an error back
  --   to the caller.
  SELECT
    @examOID = ExamOID
  FROM
    PatientExams EXAM
  WHERE
    EXAM.PatientOID = @patientOID
    AND EXAM.VisitOID = @visitOID
    AND EXAM.Entity = @entity
    AND EXAM.OrderNumber = @orderNumber
    AND EXAM.UniversalServiceCode = @uniServiceCode
    AND EXAM.OrderedForDT = @orderedForDT
    AND ISNULL(EXAM.AccessionNumber, @accItn) = @accItn

  -- Validate that the exam is found
  IF (ISNULL(@examOID, 0) = 0)
  BEGIN
    -- Exam NOT found
    RAISERROR (N'Exam NOT found, PatientOID = %d, VisitOID = %d, OrdNo = %s, SvcCd = %s.'
              ,16 -- Severity
              ,1 -- State
              ,@patientOID
              ,@visitOID
              ,@orderNumber
              ,@uniServiceCode);
  END

  IF EXISTS (SELECT 1 FROM ExamResultMap MAP INNER JOIN PatientResults RESULT ON
               RESULT.PatientResultOID = MAP.ResultOID WHERE MAP.ExamOID = @examOID)
  BEGIN
    SELECT
      @previousResultOID = resultOID
    FROM
      ExamResultMap MAP
        INNER JOIN PatientResults RESULT
          ON RESULT.PatientResultOID = MAP.ResultOID
    WHERE
      MAP.ExamOID = @examOID
  END
      -- Get the next ResultOID to use
  EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'PatientResult', @nextOID = @resultOID OUTPUT

  -- Return a single row of data to the caller that contains the data currently
  --   stored in the DB
  SELECT
    EXAM.ExamOID
  , EXAM.Entity
  , EXAM.OrderNumber
  , EXAM.UniversalServiceCode
  , EXAM.Description
  , EXAM.Priority
  , EXAM.ReasonForExam
  , EXAM.Comments
  , EXAM.OrderedOnDT
  , EXAM.OrderedForDT
  , EXAM.AccessionNumber
  , EXAM.PatientOID
  , EXAM.VisitOID
  , @resultOID AS PatientResultOID
  , @previousResultOID AS PreviousResultOID
  , RESULT.ResultStatus
  , RESULT.ResultText
  FROM
    PatientExams EXAM
      LEFT OUTER JOIN ExamResultMap MAP
        ON MAP.ExamOID = EXAM.ExamOID
      LEFT OUTER JOIN PatientResults RESULT
        ON RESULT.PatientResultOID = MAP.ResultOID
  WHERE
    EXAM.PatientOID = @patientOID
    AND EXAM.VisitOID = @visitOID
    AND EXAM.Entity = @entity
    AND EXAM.OrderNumber = @orderNumber
    AND EXAM.UniversalServiceCode = @uniServiceCode
    AND EXAM.OrderedForDT = @orderedForDT
    AND ISNULL(EXAM.AccessionNumber, @accItn) = @accItn

END
GO
