IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceSetExamData'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceSetExamData
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceSetExamData]
(
  @examOID          INT = NULL
, @patientOID       INT
, @visitOID         INT
, @entity           NVARCHAR(6)
, @uniServiceCode   NVARCHAR(12)
, @description      NVARCHAR(100)
, @priority         NVARCHAR(10)
, @orderedForDT     SMALLDATETIME
, @orderedOnDT      SMALLDATETIME
, @orderNumber      NVARCHAR(20)
, @comments         NVARCHAR(256)
, @reason           NVARCHAR(100)
)
AS
BEGIN

  DECLARE @examExists BIT

  IF (@examOID IS NULL)
  BEGIN
    SELECT @examExists = 0

      -- Get the next ExamOID to use
      EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'PatientExam', @nextOID = @examOID OUTPUT
  END
  ELSE
  BEGIN
    IF EXISTS (SELECT 1 FROM PatientExams WHERE ExamOID = @examOID AND Entity = @entity
                 AND PatientOID = @patientOID AND VisitOID = @visitOID)
    BEGIN
      SELECT @examExists = 1
    END
    ELSE
    BEGIN
      SELECT @examExists = 0
    END
  END

  IF (@examExists = 1)
  BEGIN
    UPDATE
      PatientExams
    SET
      [UniversalServiceCode] = @uniServiceCode
    , [Description]          = @description
    , [Priority]             = @priority
    , [OrderedForDT]         = @orderedForDT
    , [OrderedOnDT]          = @orderedOnDT
    , [OrderNumber]          = @orderNumber
    , [ReasonForExam]        = @reason
    , [Comments]             = @comments
    FROM
      PatientExams EXAM
    WHERE
      EXAM.ExamOID = @examOID
      AND EXAM.Entity = @entity
      AND EXAM.PatientOID = @patientOID
      AND EXAM.VisitOID = @visitOID
  END
  ELSE
  BEGIN
    INSERT PatientExams
    (
      [ExamOID]
    , [PatientOID]
    , [VisitOID]
    , [Entity]
    , [UniversalServiceCode]
    , [Description]
    , [OrderNumber]
    , [Priority]
    , [OrderedForDT]
    , [OrderedOnDT]
    , [ReasonForExam]
    , [Comments]
    )
    VALUES
    (
      @examOID
    , @patientOID
    , @visitOID
    , @entity
    , @uniServiceCode
    , @description
    , @orderNumber
    , @priority
    , @orderedForDT
    , @orderedOnDT
    , @reason
    , @comments
    )
  END

  SELECT @examOID AS ExamOID

END
GO
