IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceSetPatientContact'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceSetPatientContact
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceSetPatientContact]
(
  @patientContactOID      INT = NULL
, @patientContactType     NVARCHAR(10)
, @contactID              NVARCHAR(50)
, @lastName               NVARCHAR(50)
, @firstName              NVARCHAR(50)
, @middleName             NVARCHAR(50)
, @prefix                 NVARCHAR(50)
, @suffix                 NVARCHAR(50)
, @title                  NVARCHAR(50)
, @patientOID             INT
, @patientContactLevel    NVARCHAR(6)
, @patientContactLevelOID INT
)
AS
BEGIN

  DECLARE @patientContactExists BIT
  DECLARE @contactOID INT
  DECLARE @providerRoleOID INT

  -- Figure out if we need to add a new contact or update an existing contact for this patient.
  IF (@patientContactOID IS NULL)
  BEGIN
    SELECT @patientContactExists = 0
    EXEC hsp_GetNextOID_ByOutputVar @oidDescriptor = 'PatientContact', @nextOID = @patientContactOID OUTPUT
  END
  ELSE
  BEGIN
    SELECT @patientContactExists = 1
  END

  EXEC hsp_ResolveInterfaceContactByID_ByOutputVar
         @interfaceContactType = @patientContactType
       , @interfaceContactID   = @contactID
       , @contactOID           = @contactOID OUTPUT
       , @providerRoleOID      = @providerRoleOID OUTPUT

  IF (@patientContactExists = 1)
  BEGIN
    UPDATE
      PatientContacts
    SET
      ContactID       = @contactID
    , LastName        = @lastName
    , FirstName       = @firstName
    , MiddleName      = @middleName
    , Prefix          = @prefix
    , Suffix          = @suffix
    , Title           = @title
    , ContactOID      = @contactOID
    , ProviderRoleOID = @providerRoleOID
    FROM
      PatientContacts PC
    WHERE
      PC.PatientContactOID = @patientContactOID
      AND PC.PatientOID    = @patientOID
  END
  ELSE
  BEGIN
    INSERT PatientContacts
    (
      PatientContactOID
    , PatientContactType
    , ContactID
    , LastName
    , FirstName
    , MiddleName
    , Prefix
    , Suffix
    , Title
    , PatientOID
    , ContactOID
    , ProviderRoleOID
    )
    VALUES
    (
      @patientContactOID
    , @patientContactType
    , @contactID
    , @lastName
    , @firstName
    , @middleName
    , @prefix
    , @suffix
    , @title
    , @patientOID
    , @contactOID
    , @providerRoleOID
    )
  END

  -- Now for the relation.  Based on the contact level information create a relation
  --  at the appropriate level (ie Visit->Contact, Exam->Contact, Result->Contact)
  IF (@patientContactLevel = N'VISIT')
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM VisitContactMap WHERE VisitOID = @patientContactLevelOID
                     AND PatientContactOID = @patientContactOID)
    BEGIN
      INSERT VisitContactMap
      (
        VisitOID
      , PatientContactOID
      )
      VALUES
      (
        @patientContactLevelOID
      , @patientContactOID
      )
    END
  END
  ELSE IF (@patientContactLevel = N'EXAM')
  BEGIN
    IF NOT EXISTS (SELECT 1 FROM ExamContactMap WHERE ExamOID = @patientContactLevelOID
                     AND PatientContactOID = @patientContactOID)
    BEGIN
      INSERT ExamContactMap
      (
        ExamOID
      , PatientContactOID
      )
      VALUES
      (
        @patientContactLevelOID
      , @patientContactOID
      )
    END
  END
END
GO
