IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_InterfaceSetTransferData'))
     BEGIN
          DROP PROCEDURE dbo.hsp_InterfaceSetTransferData
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_InterfaceSetTransferData]
(
 @patientOID INT
,@visitOID   INT
,@entity     NVARCHAR(6)
,@admitClass NVARCHAR(3)
,@ns         NVARCHAR(50)
)
AS
BEGIN

  DECLARE @visitExists   BIT

  -- Make sure a visit exists for the passed identifiers
  IF EXISTS (SELECT 1 FROM PatientVisits
               WHERE PatientOID = @patientOID AND VisitOID = @visitOID AND Entity = @entity)
  BEGIN
    SELECT @visitExists = 1
  END
  ELSE
  BEGIN
    SELECT @visitExists = 0
  END

  -- If we can't find the visit we're supposed to update raise an error
  IF (@visitExists = 0)
  BEGIN
    -- Should NOT happen.  RAISEERROR to alert the caller - FATAL
    RAISERROR (N'Unable to find visit to transfer, PatientOID = %d, VisitOID = %d, Entity = %s.'
              ,0 -- Severity
              ,1 -- State
              ,@patientOID
              ,@visitOID
              ,@entity); 
  END

  -- Transfer the patient to the new location (NurseStation) and update the
  --   visit class.
  UPDATE
    PatientVisits
  SET
    AdmitClass        = @admitClass
  , NurseStation      = @ns
  FROM
    PatientVisits PV
  WHERE
    PV.VisitOID       = @visitOID
    AND PV.Entity     = @entity
    AND PV.PatientOID = @patientOID
END
GO
