IF EXISTS (SELECT * FROM sysobjects WHERE type in ('P') and id = object_id('dbo.hsp_SetInterfaceAdmissionData'))
     BEGIN
          DROP PROCEDURE dbo.hsp_SetInterfaceAdmissionData
     END
GO

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [hsp_SetInterfaceAdmissionData]
(
 @patientOID INT
,@visitOID   INT
,@entity     NVARCHAR(6)
,@lastName   NVARCHAR(50)
,@firstName  NVARCHAR(50)
,@middleName  NVARCHAR(50)
,@dob        DATETIME
,@gender     NVARCHAR(1)
,@race       NVARCHAR(1)
,@religion   NVARCHAR(50)
,@height     NVARCHAR(25)
,@weight     NVARCHAR(25)
,@isPregnant NVARCHAR(1)
,@lmp        SMALLDATETIME
,@admitDT    SMALLDATETIME
,@admitType  NVARCHAR(3)
,@admitClass NVARCHAR(3)
,@locOfSvc   NVARCHAR(50)
,@ns         NVARCHAR(50)
)
AS
BEGIN

  DECLARE @patientExists BIT
  DECLARE @visitExists   BIT

  IF EXISTS (SELECT 1 FROM Patients WHERE PatientOID = @patientOID)
  BEGIN
    SELECT @patientExists = 1
  END
  ELSE
  BEGIN
    SELECT @patientExists = 0
  END

  IF EXISTS (SELECT 1 FROM PatientVisits
               WHERE PatientOID = @patientOID AND VisitOID = @visitOID AND Entity = @entity)
  BEGIN
    SELECT @visitExists = 1
  END
  ELSE
  BEGIN
    SELECT @visitExists = 0
  END

  IF (@patientExists = 1)
  BEGIN
    UPDATE
      Patients
    SET
      LastName    = @lastName
    , FirstName   = @firstName
    , MiddleName  = @middleName
    , DateOfBirth = @dob
    , Gender      = @gender
    , Race        = @race
    , Religion    = @religion
    , Height      = @height
    , Weight      = @weight
    , IsPregnant  = @isPregnant
    , LMP         = @lmp
    FROM
      Patients P
    WHERE
      P.PatientOID = @patientOID
  END
  ELSE
  BEGIN
    INSERT Patients
    (
      PatientOID
    , LastName
    , FirstName
    , MiddleName
    , DateOfBirth
    , Gender
    , Race
    , Religion
    , Height
    , Weight
    , IsPregnant
    , LMP
    )
    VALUES
    (
      @patientOID
    , @lastName
    , @firstName
    , @middleName
    , @dob
    , @gender
    , @race
    , @religion
    , @height
    , @weight
    , @isPregnant
    , @lmp
    )
  END

  IF (@visitExists = 1)
  BEGIN
    UPDATE
      PatientVisits
    SET
      AdmitDT           = @admitDT
    , AdmitType         = @admitType
    , AdmitClass        = @admitClass
    , LocationOfService = @locOfSvc
    , NurseStation      = @ns
    FROM
      PatientVisits PV
    WHERE
      PV.VisitOID       = @visitOID
      AND PV.Entity     = @entity
      AND PV.PatientOID = @patientOID
  END
  ELSE
  BEGIN
    INSERT PatientVisits
    (
      VisitOID
    , Entity
    , AdmitDT
    , AdmitType
    , AdmitClass
    , LocationOfService
    , NurseStation
    , PatientOID
    )
    VALUES
    (
      @visitOID
    , @entity
    , @admitDT
    , @admitType
    , @admitClass
    , @locOfSvc
    , @ns
    , @patientOID
    )
  END
END
GO
